package javawrapper;


/**
 * Tests the EpiphyticEstablishmentBehaviors class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2009</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */
public class TestEpiphyticEstablishmentBehaviors
    extends ModelTestCase {

 
  /**
   * Tests validate data.
   */
  public void testValidateData() {
    //Case:  all valid.
    GUIManager oManager = null;
    String sFileName = null;
    EpiphyticEstablishmentBehaviors oEpi = null;
    TreePopulation oPop = null;
    try {

      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile1();
      oManager.InputXMLParameterFile(sFileName);
      oEpi = oManager.GetEpiphyticEstablishmentBehaviors();
      oPop = oManager.GetTreePopulation();
      oEpi.ValidateData(oPop);
    }
    catch (ModelException oErr) {
      fail("Epiphytic recruitment validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    //Case:  number of azimuth angles for GLI light is negative.
    oEpi.m_iNumAziDiv.SetValue( -10);
    try {
      oEpi.ValidateData(oPop);
      fail("Epiphytic recruitment validation failed - Case:  number of azimuth angles for GLI light is negative.");
    }
    catch (ModelException oErr) {
      ;
    }
    oEpi.m_iNumAziDiv.SetValue(10);

    //Case:  number of altitude angles for GLI light is negative.
    oEpi.m_iNumAltDiv.SetValue( -10);
    try {
      oEpi.ValidateData(oPop);
      fail("Epiphytic recruitment validation failed - Case:  number of altitude angles for GLI light is negative.");
    }
    catch (ModelException oErr) {
      ;
    }
    oEpi.m_iNumAltDiv.SetValue(10);

    //Case:  min sun angle for GLI light is zero.
    oEpi.m_fMinSunAngle.SetValue(-10);
    try {
      oEpi.ValidateData(oPop);
      fail("Epiphytic recruitment validation failed - Case:  min sun angle for GLI light is negative.");
    }
    catch (ModelException oErr) {;}
    oEpi.m_fMinSunAngle.SetValue((float)0.7);
    
    //Case: julian days not properly bounded.
    oEpi.m_iJulianDayGrowthStarts.SetValue(-10);
    try {
      oEpi.ValidateData(oPop);
      fail("Epiphytic recruitment validation failed - Case: bad Julian start day.");
    }
    catch (ModelException oErr) {;}
    oEpi.m_iJulianDayGrowthStarts.SetValue(10);
    oEpi.m_iJulianDayGrowthEnds.SetValue(-10);
    try {
      oEpi.ValidateData(oPop);
      fail("Epiphytic recruitment validation failed - Case: bad Julian end day.");
    }
    catch (ModelException oErr) {;}
    oEpi.m_iJulianDayGrowthEnds.SetValue(10);
    
    //Case: clear sky transmission coefficient less than 0.
    oEpi.m_fClearSkyTransmissionCoefficient.SetValue(-10);
    try {
      oEpi.ValidateData(oPop);
      fail("Epiphytic recruitment validation failed - Case: bad clear sky transmission coefficient.");
    }
    catch (ModelException oErr) {;}
    oEpi.m_fClearSkyTransmissionCoefficient.SetValue(10);
  }

  /**
   * Makes sure that values are properly read, and that default light settings
   * are correctly chosen.
   */
  public void testReadParFile() {
    GUIManager oManager = null;
    String sFileName = null;
    ModelData oData;
    ModelEnum oEnum;
    int i;
    try {

      oManager = new GUIManager(null);
      
      //Case: basic parameter file read.
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile1();
      oManager.InputXMLParameterFile(sFileName);
      EpiphyticEstablishmentBehaviors oEpi = oManager.GetEpiphyticEstablishmentBehaviors();
      assertEquals( ( (Float) oEpi.mp_fLightTransmissionCoefficient.GetValue().
          elementAt(0)).floatValue(), 0.0, 0.001);
      assertEquals( ( (Float) oEpi.mp_fLightTransmissionCoefficient.GetValue().
          elementAt(1)).floatValue(), 0.2, 0.001);
      assertEquals( ( (Float) oEpi.mp_fLightTransmissionCoefficient.GetValue().
          elementAt(2)).floatValue(), 0.4, 0.001);
      assertEquals(oEpi.m_fBeamFractionOfGlobalRadiation.GetValue(),
          (float) 0.5, 0.001);
      assertEquals(oEpi.m_fClearSkyTransmissionCoefficient.GetValue(),
          (float) 0.65, 0.001);
      assertEquals(oEpi.m_iJulianDayGrowthStarts.GetValue(), 105);
      assertEquals(oEpi.m_iJulianDayGrowthEnds.GetValue(), 258);
      assertEquals(oEpi.m_iSnagAgeClass1.GetValue(), 10);
      assertEquals(oEpi.m_iSnagAgeClass2.GetValue(), 17);
      assertEquals(oEpi.m_fMinSunAngle.GetValue(), (float) 0.785, 0.001);
      assertEquals(oEpi.m_iNumAltDiv.GetValue(), 12);
      assertEquals(oEpi.m_iNumAziDiv.GetValue(), 18);
      assertEquals( ( (Float) oEpi.mp_fTreeFernRecruitA.GetValue().
          elementAt(0)).floatValue(), -3.6482, 0.001);
      assertEquals( ( (Float) oEpi.mp_fTreeFernRecruitA.GetValue().
          elementAt(2)).floatValue(), -6.1567, 0.001);
      assertEquals( ( (Float) oEpi.mp_fTreeFernRecruitB.GetValue().
          elementAt(0)).floatValue(), 0.5442, 0.001);
      assertEquals( ( (Float) oEpi.mp_fTreeFernRecruitB.GetValue().
          elementAt(2)).floatValue(), 1.0008, 0.001);
      assertEquals( ( (Float) oEpi.mp_fTreeFernRecruitC.GetValue().
          elementAt(0)).floatValue(), 0.016595, 0.001);
      assertEquals( ( (Float) oEpi.mp_fTreeFernRecruitC.GetValue().
          elementAt(2)).floatValue(), 0.040162, 0.001);
      assertEquals( ( (Float) oEpi.mp_fTreeFernRecruitM.GetValue().
          elementAt(0)).floatValue(), -27.21, 0.001);
      assertEquals( ( (Float) oEpi.mp_fTreeFernRecruitM.GetValue().
          elementAt(2)).floatValue(), -16.87, 0.001);
      assertEquals( ( (Float) oEpi.mp_fTreeFernRecruitN.GetValue().
          elementAt(0)).floatValue(), 53.76, 0.001);
      assertEquals( ( (Float) oEpi.mp_fTreeFernRecruitN.GetValue().
          elementAt(2)).floatValue(), 111.86, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass1LightTransmissionCoefficient.GetValue().
          elementAt(0)).floatValue(), 0.02, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass1LightTransmissionCoefficient.GetValue().
          elementAt(1)).floatValue(), 0.42, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass1LightTransmissionCoefficient.GetValue().
          elementAt(2)).floatValue(), 0.52, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass2LightTransmissionCoefficient.GetValue().
          elementAt(0)).floatValue(), 0.03, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass2LightTransmissionCoefficient.GetValue().
          elementAt(1)).floatValue(), 0.43, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass2LightTransmissionCoefficient.GetValue().
          elementAt(2)).floatValue(), 0.53, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass3LightTransmissionCoefficient.GetValue().
          elementAt(0)).floatValue(), 0.04, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass3LightTransmissionCoefficient.GetValue().
          elementAt(1)).floatValue(), 0.44, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass3LightTransmissionCoefficient.GetValue().
          elementAt(2)).floatValue(), 0.54, 0.001);
      
      for (i = 0; i < oEpi.mp_oAllData.size(); i++) {
        oData = (ModelData) oEpi.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("Species of New Seedlings") > -1) {
          oEnum = (ModelEnum) oEpi.mp_oAllData.elementAt(i);
          assertEquals(1, oEnum.GetValue());
          break;
        }
      }
      
      new java.io.File(sFileName).delete();
                
      //Case:  only gli light data; defaults for light properly fetched.
      oManager.ClearCurrentData();
      sFileName = WriteXMLFileGLIOnly();
      oManager.InputXMLParameterFile(sFileName);
      oEpi = oManager.GetEpiphyticEstablishmentBehaviors();
      assertEquals(oEpi.m_fBeamFractionOfGlobalRadiation.GetValue(),
                   (float) 0.5, 0.001);
      assertEquals(oEpi.m_fClearSkyTransmissionCoefficient.GetValue(),
                   (float) 0.65, 0.001);
      assertEquals(oEpi.m_iJulianDayGrowthStarts.GetValue(), 105);
      assertEquals(oEpi.m_iJulianDayGrowthEnds.GetValue(), 258);
      assertEquals(oEpi.m_fMinSunAngle.GetValue(), (float) 0.785, 0.001);
      assertEquals(oEpi.m_iNumAltDiv.GetValue(), 12);
      assertEquals(oEpi.m_iNumAziDiv.GetValue(), 18);
      assertEquals(oEpi.m_iSnagAgeClass1.GetValue(), 10);
      assertEquals(oEpi.m_iSnagAgeClass2.GetValue(), 17);
      assertEquals( ( (Float) oEpi.mp_fLightTransmissionCoefficient.GetValue().
          elementAt(0)).floatValue(), 0.08, 0.001);
      assertEquals( ( (Float) oEpi.mp_fLightTransmissionCoefficient.GetValue().
          elementAt(1)).floatValue(), 0.09, 0.001);
      assertEquals( ( (Float) oEpi.mp_fLightTransmissionCoefficient.GetValue().
          elementAt(2)).floatValue(), 0.10, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass1LightTransmissionCoefficient.GetValue().
          elementAt(0)).floatValue(), 0.02, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass1LightTransmissionCoefficient.GetValue().
          elementAt(1)).floatValue(), 0.42, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass1LightTransmissionCoefficient.GetValue().
          elementAt(2)).floatValue(), 0.52, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass2LightTransmissionCoefficient.GetValue().
          elementAt(0)).floatValue(), 0.03, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass2LightTransmissionCoefficient.GetValue().
          elementAt(1)).floatValue(), 0.43, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass2LightTransmissionCoefficient.GetValue().
          elementAt(2)).floatValue(), 0.53, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass3LightTransmissionCoefficient.GetValue().
          elementAt(0)).floatValue(), 0.04, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass3LightTransmissionCoefficient.GetValue().
          elementAt(1)).floatValue(), 0.44, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass3LightTransmissionCoefficient.GetValue().
          elementAt(2)).floatValue(), 0.54, 0.001);

      new java.io.File(sFileName).delete();

      //Case:  only quadrat light data.
      oManager.ClearCurrentData();
      sFileName = WriteXMLFileQuadratOnly();
      oManager.InputXMLParameterFile(sFileName);
      oEpi = oManager.GetEpiphyticEstablishmentBehaviors();
      assertEquals(oEpi.m_fBeamFractionOfGlobalRadiation.GetValue(),
                   (float) 0.5, 0.001);
      assertEquals(oEpi.m_fClearSkyTransmissionCoefficient.GetValue(),
                   (float) 0.65, 0.001);
      assertEquals(oEpi.m_iJulianDayGrowthStarts.GetValue(), 105);
      assertEquals(oEpi.m_iJulianDayGrowthEnds.GetValue(), 258);
      assertEquals(oEpi.m_fMinSunAngle.GetValue(), (float) 0.785, 0.001);
      assertEquals(oEpi.m_iNumAltDiv.GetValue(), 12);
      assertEquals(oEpi.m_iNumAziDiv.GetValue(), 18);
      assertEquals(oEpi.m_iSnagAgeClass1.GetValue(), 10);
      assertEquals(oEpi.m_iSnagAgeClass2.GetValue(), 17);
      assertEquals( ( (Float) oEpi.mp_fLightTransmissionCoefficient.GetValue().
          elementAt(0)).floatValue(), 0.08, 0.001);
      assertEquals( ( (Float) oEpi.mp_fLightTransmissionCoefficient.GetValue().
          elementAt(1)).floatValue(), 0.09, 0.001);
      assertEquals( ( (Float) oEpi.mp_fLightTransmissionCoefficient.GetValue().
          elementAt(2)).floatValue(), 0.10, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass1LightTransmissionCoefficient.GetValue().
          elementAt(0)).floatValue(), 0.02, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass1LightTransmissionCoefficient.GetValue().
          elementAt(1)).floatValue(), 0.42, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass1LightTransmissionCoefficient.GetValue().
          elementAt(2)).floatValue(), 0.52, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass2LightTransmissionCoefficient.GetValue().
          elementAt(0)).floatValue(), 0.03, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass2LightTransmissionCoefficient.GetValue().
          elementAt(1)).floatValue(), 0.43, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass2LightTransmissionCoefficient.GetValue().
          elementAt(2)).floatValue(), 0.53, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass3LightTransmissionCoefficient.GetValue().
          elementAt(0)).floatValue(), 0.04, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass3LightTransmissionCoefficient.GetValue().
          elementAt(1)).floatValue(), 0.44, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass3LightTransmissionCoefficient.GetValue().
          elementAt(2)).floatValue(), 0.54, 0.001);

      new java.io.File(sFileName).delete();



      //Case:  only gli map data.
      oManager.ClearCurrentData();
      sFileName = WriteXMLFileMapOnly();
      oManager.InputXMLParameterFile(sFileName);
      oEpi = oManager.GetEpiphyticEstablishmentBehaviors();
      assertEquals(oEpi.m_fBeamFractionOfGlobalRadiation.GetValue(),
                   (float) 0.5, 0.001);
      assertEquals(oEpi.m_fClearSkyTransmissionCoefficient.GetValue(),
                   (float) 0.65, 0.001);
      assertEquals(oEpi.m_iJulianDayGrowthStarts.GetValue(), 105);
      assertEquals(oEpi.m_iJulianDayGrowthEnds.GetValue(), 258);
      assertEquals(oEpi.m_fMinSunAngle.GetValue(), (float) 0.785, 0.001);
      assertEquals(oEpi.m_iNumAltDiv.GetValue(), 12);
      assertEquals(oEpi.m_iNumAziDiv.GetValue(), 18);
      assertEquals(oEpi.m_iSnagAgeClass1.GetValue(), 10);
      assertEquals(oEpi.m_iSnagAgeClass2.GetValue(), 17);
      assertEquals( ( (Float) oEpi.mp_fLightTransmissionCoefficient.GetValue().
          elementAt(0)).floatValue(), 0.08, 0.001);
      assertEquals( ( (Float) oEpi.mp_fLightTransmissionCoefficient.GetValue().
          elementAt(1)).floatValue(), 0.09, 0.001);
      assertEquals( ( (Float) oEpi.mp_fLightTransmissionCoefficient.GetValue().
          elementAt(2)).floatValue(), 0.10, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass1LightTransmissionCoefficient.GetValue().
          elementAt(0)).floatValue(), 0.02, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass1LightTransmissionCoefficient.GetValue().
          elementAt(1)).floatValue(), 0.42, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass1LightTransmissionCoefficient.GetValue().
          elementAt(2)).floatValue(), 0.52, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass2LightTransmissionCoefficient.GetValue().
          elementAt(0)).floatValue(), 0.03, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass2LightTransmissionCoefficient.GetValue().
          elementAt(1)).floatValue(), 0.43, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass2LightTransmissionCoefficient.GetValue().
          elementAt(2)).floatValue(), 0.53, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass3LightTransmissionCoefficient.GetValue().
          elementAt(0)).floatValue(), 0.04, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass3LightTransmissionCoefficient.GetValue().
          elementAt(1)).floatValue(), 0.44, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass3LightTransmissionCoefficient.GetValue().
          elementAt(2)).floatValue(), 0.54, 0.001);

      new java.io.File(sFileName).delete();

      //Case:  only gli points data.
      oManager.ClearCurrentData();
      sFileName = WriteXMLFilePointsOnly();
      oManager.InputXMLParameterFile(sFileName);
      oEpi = oManager.GetEpiphyticEstablishmentBehaviors();
      assertEquals(oEpi.m_fBeamFractionOfGlobalRadiation.GetValue(),
                   (float) 0.5, 0.001);
      assertEquals(oEpi.m_fClearSkyTransmissionCoefficient.GetValue(),
                   (float) 0.65, 0.001);
      assertEquals(oEpi.m_iJulianDayGrowthStarts.GetValue(), 105);
      assertEquals(oEpi.m_iJulianDayGrowthEnds.GetValue(), 258);
      assertEquals(oEpi.m_fMinSunAngle.GetValue(), (float) 0.785, 0.001);
      assertEquals(oEpi.m_iNumAltDiv.GetValue(), 12);
      assertEquals(oEpi.m_iNumAziDiv.GetValue(), 18);
      assertEquals(oEpi.m_iSnagAgeClass1.GetValue(), 10);
      assertEquals(oEpi.m_iSnagAgeClass2.GetValue(), 17);
      assertEquals( ( (Float) oEpi.mp_fLightTransmissionCoefficient.GetValue().
          elementAt(0)).floatValue(), 0.08, 0.001);
      assertEquals( ( (Float) oEpi.mp_fLightTransmissionCoefficient.GetValue().
          elementAt(1)).floatValue(), 0.09, 0.001);
      assertEquals( ( (Float) oEpi.mp_fLightTransmissionCoefficient.GetValue().
          elementAt(2)).floatValue(), 0.10, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass1LightTransmissionCoefficient.GetValue().
          elementAt(0)).floatValue(), 0.02, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass1LightTransmissionCoefficient.GetValue().
          elementAt(1)).floatValue(), 0.42, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass1LightTransmissionCoefficient.GetValue().
          elementAt(2)).floatValue(), 0.52, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass2LightTransmissionCoefficient.GetValue().
          elementAt(0)).floatValue(), 0.03, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass2LightTransmissionCoefficient.GetValue().
          elementAt(1)).floatValue(), 0.43, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass2LightTransmissionCoefficient.GetValue().
          elementAt(2)).floatValue(), 0.53, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass3LightTransmissionCoefficient.GetValue().
          elementAt(0)).floatValue(), 0.04, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass3LightTransmissionCoefficient.GetValue().
          elementAt(1)).floatValue(), 0.44, 0.001);
      assertEquals( ( (Float) oEpi.mp_fSnagClass3LightTransmissionCoefficient.GetValue().
          elementAt(2)).floatValue(), 0.54, 0.001);
      
      new java.io.File(sFileName).delete();
      
      //Case: epiphytic recruitment plus light.
      oManager.ClearCurrentData();
      sFileName = WriteXMLFileEpiPlusGLI();
      oManager.InputXMLParameterFile(sFileName);
      oEpi = oManager.GetEpiphyticEstablishmentBehaviors();
      assertEquals( ( (Float) oEpi.mp_fLightTransmissionCoefficient.GetValue().
          elementAt(0)).floatValue(), 0.0, 0.001);
      assertEquals( ( (Float) oEpi.mp_fLightTransmissionCoefficient.GetValue().
          elementAt(1)).floatValue(), 0.2, 0.001);
      assertEquals( ( (Float) oEpi.mp_fLightTransmissionCoefficient.GetValue().
          elementAt(2)).floatValue(), 0.4, 0.001);
      assertEquals(oEpi.m_fBeamFractionOfGlobalRadiation.GetValue(),
          (float) 0.4, 0.001);
      assertEquals(oEpi.m_fClearSkyTransmissionCoefficient.GetValue(),
          (float) 0.64, 0.001);
      assertEquals(oEpi.m_iJulianDayGrowthStarts.GetValue(), 104);
      assertEquals(oEpi.m_iJulianDayGrowthEnds.GetValue(), 257);
      assertEquals(oEpi.m_fMinSunAngle.GetValue(), (float) 0.784, 0.001);
      assertEquals(oEpi.m_iNumAltDiv.GetValue(), 11);
      assertEquals(oEpi.m_iNumAziDiv.GetValue(), 17);
      
      new java.io.File(sFileName).delete();

    }
    catch (ModelException oErr) {
      fail("Epiphytic recruitment validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Tests that the correct parameters are displayed for each behavior.
   */
  public void testFormatDataForDisplay() {
    try {
      GUIManager oManager = new GUIManager(null);
      TreePopulation oPop;
      EpiphyticEstablishmentBehaviors oEpi = null;
      String[] p_sExpected;
      Behavior oBeh;

      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteNoEpiphyticEstablishmentXMLFile1());
      oEpi = oManager.GetEpiphyticEstablishmentBehaviors();
      oPop = oManager.GetTreePopulation();

      //Case:  no behaviors are enabled.
      assertEquals(null, oEpi.FormatDataForDisplay(oPop));

      //Case:  tree fern behavior is enabled.
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteNoEpiphyticEstablishmentXMLFile1());
      oEpi = oManager.GetEpiphyticEstablishmentBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oEpi.GetBehaviorByKey("Tree Fern Establishment");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Tree Fern Establishment Seedling Prob \"a\"",
          "Tree Fern Establishment Seedling Prob \"b\"",
          "Tree Fern Establishment Seedling Prob \"c\"",
          "Tree Fern Establishment Seedling Height \"m\"",
          "Tree Fern Establishment Seedling Height \"n\"",
          "Beam Fraction of Global Radiation",
          "Clear Sky Transmission Coefficient",
          "Snag Age Class 1 Amount Canopy Light Transmission (0-1)",
          "Snag Age Class 2 Amount Canopy Light Transmission (0-1)",
          "Snag Age Class 3 Amount Canopy Light Transmission (0-1)",
          "Amount Canopy Light Transmission (0-1)",
          "Minimum Solar Angle for GLI Calculations, in rad",
          "Upper Age (Yrs) of Snag Light Transmission Class 1",
          "Upper Age (Yrs) of Snag Light Transmission Class 2",
          "Tree Fern Establishment Species of New Seedlings",
          "Number of Azimuth Sky Divisions for GLI Light Calculations",
          "Number of Altitude Sky Divisions for GLI Light Calculations",
          "First Day of Growing Season for GLI Light Calculations",
          "Last Day of Growing Season for GLI Light Calculations"
      };
      TestTable(oEpi.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  all are enabled.
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteNoEpiphyticEstablishmentXMLFile1());
      oEpi = oManager.GetEpiphyticEstablishmentBehaviors();
      oPop = oManager.GetTreePopulation();
      int i;
      for (i = 0; i < oEpi.mp_oChildBehaviors.length; i++) {
        oEpi.mp_oChildBehaviors[i].m_bIsEnabled = true;
        oEpi.mp_oChildBehaviors[i].AddSpeciesTypeCombo(
          		new SpeciesTypeCombo(0, 3, oPop));
      }
      p_sExpected = new String[] {
          "Tree Fern Establishment Seedling Prob \"a\"",
          "Tree Fern Establishment Seedling Prob \"b\"",
          "Tree Fern Establishment Seedling Prob \"c\"",
          "Tree Fern Establishment Seedling Height \"m\"",
          "Tree Fern Establishment Seedling Height \"n\"",
          "Beam Fraction of Global Radiation",
          "Clear Sky Transmission Coefficient",
          "Snag Age Class 1 Amount Canopy Light Transmission (0-1)",
          "Snag Age Class 2 Amount Canopy Light Transmission (0-1)",
          "Snag Age Class 3 Amount Canopy Light Transmission (0-1)",
          "Amount Canopy Light Transmission (0-1)",
          "Minimum Solar Angle for GLI Calculations, in rad",
          "Upper Age (Yrs) of Snag Light Transmission Class 1",
          "Upper Age (Yrs) of Snag Light Transmission Class 2",
          "Tree Fern Establishment Species of New Seedlings",
          "Number of Azimuth Sky Divisions for GLI Light Calculations",
          "Number of Altitude Sky Divisions for GLI Light Calculations",
          "First Day of Growing Season for GLI Light Calculations",
          "Last Day of Growing Season for GLI Light Calculations"
      };
      TestTable(oEpi.FormatDataForDisplay(oPop), p_sExpected);

      System.out.println("FormatDataForDisplay succeeded for epiphytic recruitment.");
    }
    catch (ModelException oErr) {
      fail("Caught error with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Writes a file with basic epiphytic recruitment settings.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteXMLFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>adultstochasticmort</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>juvstochasticmort</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Epiphytic Establishment</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>removedead</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<epiphyticEstablishment>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0</li_lecVal>");
    oOut.write("<li_lecVal species=\"Species_2\">0.2</li_lecVal>");
    oOut.write("<li_lecVal species=\"Species_3\">0.4</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("<ep_epiphyticA>");
    oOut.write("<ep_eaVal species=\"Species_1\">-3.6482</ep_eaVal>");
    oOut.write("<ep_eaVal species=\"Species_3\">-6.1567</ep_eaVal>");
    oOut.write("</ep_epiphyticA>");
    oOut.write("<ep_epiphyticB>");
    oOut.write("<ep_ebVal species=\"Species_1\">0.5442</ep_ebVal>");
    oOut.write("<ep_ebVal species=\"Species_3\">1.0008</ep_ebVal>");
    oOut.write("</ep_epiphyticB>");
    oOut.write("<ep_epiphyticC>");
    oOut.write("<ep_ecVal species=\"Species_1\">0.016595</ep_ecVal>");
    oOut.write("<ep_ecVal species=\"Species_3\">0.040162</ep_ecVal>");
    oOut.write("</ep_epiphyticC>");
    oOut.write("<ep_epiphyticM>");
    oOut.write("<ep_emVal species=\"Species_1\">-27.21</ep_emVal>");
    oOut.write("<ep_emVal species=\"Species_3\">-16.87</ep_emVal>");
    oOut.write("</ep_epiphyticM>");
    oOut.write("<ep_epiphyticN>");
    oOut.write("<ep_enVal species=\"Species_1\">53.76</ep_enVal>");
    oOut.write("<ep_enVal species=\"Species_3\">111.86</ep_enVal>");
    oOut.write("</ep_epiphyticN>");
    oOut.write("<ep_epiphyticSeedlingSpecies>1</ep_epiphyticSeedlingSpecies>");
    oOut.write("<li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_s1lecVal species=\"Species_1\">0.02</li_s1lecVal>");
    oOut.write("<li_s1lecVal species=\"Species_2\">0.42</li_s1lecVal>");
    oOut.write("<li_s1lecVal species=\"Species_3\">0.52</li_s1lecVal>");
    oOut.write("</li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_s2lecVal species=\"Species_1\">0.03</li_s2lecVal>");
    oOut.write("<li_s2lecVal species=\"Species_2\">0.43</li_s2lecVal>");
    oOut.write("<li_s2lecVal species=\"Species_3\">0.53</li_s2lecVal>");
    oOut.write("</li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_snag3LightExtinctionCoefficient>");
    oOut.write("<li_s3lecVal species=\"Species_1\">0.04</li_s3lecVal>");
    oOut.write("<li_s3lecVal species=\"Species_2\">0.44</li_s3lecVal>");
    oOut.write("<li_s3lecVal species=\"Species_3\">0.54</li_s3lecVal>");
    oOut.write("</li_snag3LightExtinctionCoefficient>");
    oOut.write("<li_snagAgeClass1>10</li_snagAgeClass1>");
    oOut.write("<li_snagAgeClass2>17</li_snagAgeClass2>");
    oOut.write("</epiphyticEstablishment>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }
  
  /**
   * Writes a file with only gli light settings.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteXMLFileGLIOnly() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>glilight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>radiallight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<gliLight>");
    oOut.write("<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.08</li_lecVal>");
    oOut.write("<li_lecVal species=\"Species_2\">0.09</li_lecVal>");
    oOut.write("<li_lecVal species=\"Species_3\">0.10</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("<li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_s1lecVal species=\"Species_1\">0.02</li_s1lecVal>");
    oOut.write("<li_s1lecVal species=\"Species_2\">0.42</li_s1lecVal>");
    oOut.write("<li_s1lecVal species=\"Species_3\">0.52</li_s1lecVal>");
    oOut.write("</li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_s2lecVal species=\"Species_1\">0.03</li_s2lecVal>");
    oOut.write("<li_s2lecVal species=\"Species_2\">0.43</li_s2lecVal>");
    oOut.write("<li_s2lecVal species=\"Species_3\">0.53</li_s2lecVal>");
    oOut.write("</li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_snag3LightExtinctionCoefficient>");
    oOut.write("<li_s3lecVal species=\"Species_1\">0.04</li_s3lecVal>");
    oOut.write("<li_s3lecVal species=\"Species_2\">0.44</li_s3lecVal>");
    oOut.write("<li_s3lecVal species=\"Species_3\">0.54</li_s3lecVal>");
    oOut.write("</li_snag3LightExtinctionCoefficient>");
    oOut.write("<li_snagAgeClass1>10</li_snagAgeClass1>");
    oOut.write("<li_snagAgeClass2>17</li_snagAgeClass2>");
    oOut.write("</gliLight>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file with only quadrat light settings.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteXMLFileQuadratOnly() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>quadratlight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<quadratGliLight>");
    oOut.write("<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.08</li_lecVal>");
    oOut.write("<li_lecVal species=\"Species_2\">0.09</li_lecVal>");
    oOut.write("<li_lecVal species=\"Species_3\">0.10</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("<li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_s1lecVal species=\"Species_1\">0.02</li_s1lecVal>");
    oOut.write("<li_s1lecVal species=\"Species_2\">0.42</li_s1lecVal>");
    oOut.write("<li_s1lecVal species=\"Species_3\">0.52</li_s1lecVal>");
    oOut.write("</li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_s2lecVal species=\"Species_1\">0.03</li_s2lecVal>");
    oOut.write("<li_s2lecVal species=\"Species_2\">0.43</li_s2lecVal>");
    oOut.write("<li_s2lecVal species=\"Species_3\">0.53</li_s2lecVal>");
    oOut.write("</li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_snag3LightExtinctionCoefficient>");
    oOut.write("<li_s3lecVal species=\"Species_1\">0.04</li_s3lecVal>");
    oOut.write("<li_s3lecVal species=\"Species_2\">0.44</li_s3lecVal>");
    oOut.write("<li_s3lecVal species=\"Species_3\">0.54</li_s3lecVal>");
    oOut.write("</li_snag3LightExtinctionCoefficient>");
    oOut.write("<li_snagAgeClass1>10</li_snagAgeClass1>");
    oOut.write("<li_snagAgeClass2>17</li_snagAgeClass2>");
    oOut.write("</quadratGliLight>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }


  /**
   * Writes a file with only gli map settings.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteXMLFileMapOnly() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    oOut.write("<grid gridName=\"GLI Map\">");
    oOut.write("<ma_floatCodes>");
    oOut.write("<ma_floatCode label=\"GLI\">0</ma_floatCode>");
    oOut.write("</ma_floatCodes>");
    oOut.write("<ma_plotLenX>100.0</ma_plotLenX>");
    oOut.write("<ma_plotLenY>100.0</ma_plotLenY>");
    oOut.write("<ma_lengthXCells>10.0</ma_lengthXCells>");
    oOut.write("<ma_lengthYCells>10.0</ma_lengthYCells>");
    oOut.write("</grid>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>GLI Map Creator</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<gliMap>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.08</li_lecVal>");
    oOut.write("<li_lecVal species=\"Species_2\">0.09</li_lecVal>");
    oOut.write("<li_lecVal species=\"Species_3\">0.10</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("<li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_s1lecVal species=\"Species_1\">0.02</li_s1lecVal>");
    oOut.write("<li_s1lecVal species=\"Species_2\">0.42</li_s1lecVal>");
    oOut.write("<li_s1lecVal species=\"Species_3\">0.52</li_s1lecVal>");
    oOut.write("</li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_s2lecVal species=\"Species_1\">0.03</li_s2lecVal>");
    oOut.write("<li_s2lecVal species=\"Species_2\">0.43</li_s2lecVal>");
    oOut.write("<li_s2lecVal species=\"Species_3\">0.53</li_s2lecVal>");
    oOut.write("</li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_snag3LightExtinctionCoefficient>");
    oOut.write("<li_s3lecVal species=\"Species_1\">0.04</li_s3lecVal>");
    oOut.write("<li_s3lecVal species=\"Species_2\">0.44</li_s3lecVal>");
    oOut.write("<li_s3lecVal species=\"Species_3\">0.54</li_s3lecVal>");
    oOut.write("</li_snag3LightExtinctionCoefficient>");
    oOut.write("<li_snagAgeClass1>10</li_snagAgeClass1>");
    oOut.write("<li_snagAgeClass2>17</li_snagAgeClass2>");
    oOut.write("<li_mapLightHeight>2</li_mapLightHeight>");
    oOut.write("</gliMap>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file with only gli point settings.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteXMLFilePointsOnly() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>GLI Point Creator</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<gliPoint>");
    oOut.write("<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.08</li_lecVal>");
    oOut.write("<li_lecVal species=\"Species_2\">0.09</li_lecVal>");
    oOut.write("<li_lecVal species=\"Species_3\">0.10</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("<li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_s1lecVal species=\"Species_1\">0.02</li_s1lecVal>");
    oOut.write("<li_s1lecVal species=\"Species_2\">0.42</li_s1lecVal>");
    oOut.write("<li_s1lecVal species=\"Species_3\">0.52</li_s1lecVal>");
    oOut.write("</li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_s2lecVal species=\"Species_1\">0.03</li_s2lecVal>");
    oOut.write("<li_s2lecVal species=\"Species_2\">0.43</li_s2lecVal>");
    oOut.write("<li_s2lecVal species=\"Species_3\">0.53</li_s2lecVal>");
    oOut.write("</li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_snag3LightExtinctionCoefficient>");
    oOut.write("<li_s3lecVal species=\"Species_1\">0.04</li_s3lecVal>");
    oOut.write("<li_s3lecVal species=\"Species_2\">0.44</li_s3lecVal>");
    oOut.write("<li_s3lecVal species=\"Species_3\">0.54</li_s3lecVal>");
    oOut.write("</li_snag3LightExtinctionCoefficient>");
    oOut.write("<li_snagAgeClass1>10</li_snagAgeClass1>");
    oOut.write("<li_snagAgeClass2>17</li_snagAgeClass2>");
    oOut.write("<li_GLIPointsFilename>test.txt</li_GLIPointsFilename>");
    oOut.write("<li_GLIPoint x=\"75\" y=\"80\" h=\"15\"/>");
    oOut.write("<li_GLIPoint x=\"50\" y=\"50\" h=\"12\"/>");
    oOut.write("<li_GLIPoint x=\"90\" y=\"13\" h=\"5\"/>");
    oOut.write("</gliPoint>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }
  
  /**
   * Writes a file with basic epiphytic recruitment settings, and GLI as well.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteXMLFileEpiPlusGLI() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);
    WriteValidateFilePart1(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>glilight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Epiphytic Establishment</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>removedead</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_3\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<gliLight>");
    oOut.write("<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.08</li_lecVal>");
    oOut.write("<li_lecVal species=\"Species_2\">0.09</li_lecVal>");
    oOut.write("<li_lecVal species=\"Species_3\">0.10</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("<li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_s1lecVal species=\"Species_1\">0.02</li_s1lecVal>");
    oOut.write("<li_s1lecVal species=\"Species_2\">0.42</li_s1lecVal>");
    oOut.write("<li_s1lecVal species=\"Species_3\">0.52</li_s1lecVal>");
    oOut.write("</li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_s2lecVal species=\"Species_1\">0.03</li_s2lecVal>");
    oOut.write("<li_s2lecVal species=\"Species_2\">0.43</li_s2lecVal>");
    oOut.write("<li_s2lecVal species=\"Species_3\">0.53</li_s2lecVal>");
    oOut.write("</li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_snag3LightExtinctionCoefficient>");
    oOut.write("<li_s3lecVal species=\"Species_1\">0.04</li_s3lecVal>");
    oOut.write("<li_s3lecVal species=\"Species_2\">0.44</li_s3lecVal>");
    oOut.write("<li_s3lecVal species=\"Species_3\">0.54</li_s3lecVal>");
    oOut.write("</li_snag3LightExtinctionCoefficient>");
    oOut.write("<li_snagAgeClass1>10</li_snagAgeClass1>");
    oOut.write("<li_snagAgeClass2>17</li_snagAgeClass2>");
    oOut.write("</gliLight>");
    oOut.write("<epiphyticEstablishment>");
    oOut.write("<li_beamFractGlobalRad>0.4</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.64</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>104</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>257</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.784</li_minSunAngle>");
    oOut.write("<li_numAltGrids>11</li_numAltGrids>");
    oOut.write("<li_numAziGrids>17</li_numAziGrids>");
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0</li_lecVal>");
    oOut.write("<li_lecVal species=\"Species_2\">0.2</li_lecVal>");
    oOut.write("<li_lecVal species=\"Species_3\">0.4</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("<ep_epiphyticA>");
    oOut.write("<ep_eaVal species=\"Species_1\">-3.6482</ep_eaVal>");
    oOut.write("<ep_eaVal species=\"Species_3\">-6.1567</ep_eaVal>");
    oOut.write("</ep_epiphyticA>");
    oOut.write("<ep_epiphyticB>");
    oOut.write("<ep_ebVal species=\"Species_1\">0.5442</ep_ebVal>");
    oOut.write("<ep_ebVal species=\"Species_3\">1.0008</ep_ebVal>");
    oOut.write("</ep_epiphyticB>");
    oOut.write("<ep_epiphyticC>");
    oOut.write("<ep_ecVal species=\"Species_1\">0.016595</ep_ecVal>");
    oOut.write("<ep_ecVal species=\"Species_3\">0.040162</ep_ecVal>");
    oOut.write("</ep_epiphyticC>");
    oOut.write("<ep_epiphyticM>");
    oOut.write("<ep_emVal species=\"Species_1\">-27.21</ep_emVal>");
    oOut.write("<ep_emVal species=\"Species_3\">-16.87</ep_emVal>");
    oOut.write("</ep_epiphyticM>");
    oOut.write("<ep_epiphyticN>");
    oOut.write("<ep_enVal species=\"Species_1\">53.76</ep_enVal>");
    oOut.write("<ep_enVal species=\"Species_3\">111.86</ep_enVal>");
    oOut.write("</ep_epiphyticN>");
    oOut.write("<ep_epiphyticSeedlingSpecies>1</ep_epiphyticSeedlingSpecies>");
    oOut.write("<li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_s1lecVal species=\"Species_1\">0.02</li_s1lecVal>");
    oOut.write("<li_s1lecVal species=\"Species_2\">0.42</li_s1lecVal>");
    oOut.write("<li_s1lecVal species=\"Species_3\">0.52</li_s1lecVal>");
    oOut.write("</li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_s2lecVal species=\"Species_1\">0.03</li_s2lecVal>");
    oOut.write("<li_s2lecVal species=\"Species_2\">0.43</li_s2lecVal>");
    oOut.write("<li_s2lecVal species=\"Species_3\">0.53</li_s2lecVal>");
    oOut.write("</li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_snag3LightExtinctionCoefficient>");
    oOut.write("<li_s3lecVal species=\"Species_1\">0.04</li_s3lecVal>");
    oOut.write("<li_s3lecVal species=\"Species_2\">0.44</li_s3lecVal>");
    oOut.write("<li_s3lecVal species=\"Species_3\">0.54</li_s3lecVal>");
    oOut.write("</li_snag3LightExtinctionCoefficient>");
    oOut.write("<li_snagAgeClass1>10</li_snagAgeClass1>");
    oOut.write("<li_snagAgeClass2>17</li_snagAgeClass2>");
    oOut.write("</epiphyticEstablishment>");
    oOut.write("</paramFile>");;

    oOut.close();
    return sFileName;
  }

  /**
   * Writes the beginning of test parameter files.
   * @param oOut FileWriter Filestream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteValidateFilePart1(java.io.FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>3</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>100.0</plot_lenX>");
    oOut.write("<plot_lenY>100.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("<tr_species speciesName=\"Species_3\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_3\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("<tr_maxSeedlingHeight>");
    oOut.write("<tr_mshVal species=\"Species_1\">1</tr_mshVal>");
    oOut.write("<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>");
    oOut.write("<tr_mshVal species=\"Species_3\">1</tr_mshVal>");
    oOut.write("</tr_maxSeedlingHeight>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_3\">39.48</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_3\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_3\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">1</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">1</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_3\">1</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_interceptDiam10ToDBH>");
    oOut.write("<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>");
    oOut.write("<tr_idtdVal species=\"Species_2\">0</tr_idtdVal>");
    oOut.write("<tr_idtdVal species=\"Species_3\">0</tr_idtdVal>");
    oOut.write("</tr_interceptDiam10ToDBH>");
    oOut.write("<tr_whatSeedlingHeightDiam>");
    oOut.write("<tr_wsehdVal species=\"Species_1\">1</tr_wsehdVal>");
    oOut.write("<tr_wsehdVal species=\"Species_2\">1</tr_wsehdVal>");
    oOut.write("<tr_wsehdVal species=\"Species_3\">1</tr_wsehdVal>");
    oOut.write("</tr_whatSeedlingHeightDiam>");
    oOut.write("<tr_whatSaplingHeightDiam>");
    oOut.write("<tr_wsahdVal species=\"Species_1\">1</tr_wsahdVal>");
    oOut.write("<tr_wsahdVal species=\"Species_2\">1</tr_wsahdVal>");
    oOut.write("<tr_wsahdVal species=\"Species_3\">1</tr_wsahdVal>");
    oOut.write("</tr_whatSaplingHeightDiam>");
    oOut.write("<tr_whatAdultHeightDiam>");
    oOut.write("<tr_wahdVal species=\"Species_1\">1</tr_wahdVal>");
    oOut.write("<tr_wahdVal species=\"Species_2\">1</tr_wahdVal>");
    oOut.write("<tr_wahdVal species=\"Species_3\">1</tr_wahdVal>");
    oOut.write("</tr_whatAdultHeightDiam>");
    oOut.write("<tr_whatAdultCrownRadDiam>");
    oOut.write("<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>");
    oOut.write("<tr_wacrdVal species=\"Species_2\">0</tr_wacrdVal>");
    oOut.write("<tr_wacrdVal species=\"Species_3\">0</tr_wacrdVal>");
    oOut.write("</tr_whatAdultCrownRadDiam>");
    oOut.write("<tr_whatAdultCrownHeightHeight>");
    oOut.write("<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>");
    oOut.write("<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>");
    oOut.write("<tr_wachhVal species=\"Species_3\">0</tr_wachhVal>");
    oOut.write("</tr_whatAdultCrownHeightHeight>");
    oOut.write("<tr_whatSaplingCrownRadDiam>");
    oOut.write("<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>");
    oOut.write("<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>");
    oOut.write("<tr_wscrdVal species=\"Species_3\">0</tr_wscrdVal>");
    oOut.write("</tr_whatSaplingCrownRadDiam>");
    oOut.write("<tr_whatSaplingCrownHeightHeight>");
    oOut.write("<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>");
    oOut.write("<tr_wschhVal species=\"Species_2\">0</tr_wschhVal>");
    oOut.write("<tr_wschhVal species=\"Species_3\">0</tr_wschhVal>");
    oOut.write("</tr_whatSaplingCrownHeightHeight>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.589</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.589</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_3\">0.589</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_adultLinearSlope>");
    oOut.write("<tr_alsVal species=\"Species_1\">0.1299</tr_alsVal>");
    oOut.write("<tr_alsVal species=\"Species_2\">0.1299</tr_alsVal>");
    oOut.write("<tr_alsVal species=\"Species_3\">0.1299</tr_alsVal>");
    oOut.write("</tr_adultLinearSlope>");
    oOut.write("<tr_adultLinearIntercept>");
    oOut.write("<tr_aliVal species=\"Species_1\">0.1</tr_aliVal>");
    oOut.write("<tr_aliVal species=\"Species_2\">0.1</tr_aliVal>");
    oOut.write("<tr_aliVal species=\"Species_3\">0.1</tr_aliVal>");
    oOut.write("</tr_adultLinearIntercept>");
    oOut.write("<tr_saplingLinearSlope>");
    oOut.write("<tr_salsVal species=\"Species_1\">0.1299</tr_salsVal>");
    oOut.write("<tr_salsVal species=\"Species_2\">0.1299</tr_salsVal>");
    oOut.write("<tr_salsVal species=\"Species_3\">0.1299</tr_salsVal>");
    oOut.write("</tr_saplingLinearSlope>");
    oOut.write("<tr_saplingLinearIntercept>");
    oOut.write("<tr_saliVal species=\"Species_1\">0.1</tr_saliVal>");
    oOut.write("<tr_saliVal species=\"Species_2\">0.1</tr_saliVal>");
    oOut.write("<tr_saliVal species=\"Species_3\">0.1</tr_saliVal>");
    oOut.write("</tr_saplingLinearIntercept>");
    oOut.write("<tr_seedlingLinearSlope>");
    oOut.write("<tr_selsVal species=\"Species_1\">0.1299</tr_selsVal>");
    oOut.write("<tr_selsVal species=\"Species_2\">0.1299</tr_selsVal>");
    oOut.write("<tr_selsVal species=\"Species_3\">0.1299</tr_selsVal>");
    oOut.write("</tr_seedlingLinearSlope>");
    oOut.write("<tr_seedlingLinearIntercept>");
    oOut.write("<tr_seliVal species=\"Species_1\">0.1</tr_seliVal>");
    oOut.write("<tr_seliVal species=\"Species_2\">0.1</tr_seliVal>");
    oOut.write("<tr_seliVal species=\"Species_3\">0.1</tr_seliVal>");
    oOut.write("</tr_seedlingLinearIntercept>");
    oOut.write("</allometry>");
  }

  /**
   * Writes a file with no epiphytic recruitment settings.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteNoEpiphyticEstablishmentXMLFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>densityselfthinning</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<mortality>");
    oOut.write("<mo_selfThinRadius>");
    oOut.write("<mo_strVal species=\"Species_1\">9</mo_strVal>");
    oOut.write("</mo_selfThinRadius>");
    oOut.write("<mo_minDensityForMort>");
    oOut.write("<mo_mdfmVal species=\"Species_1\">0</mo_mdfmVal>");
    oOut.write("</mo_minDensityForMort>");
    oOut.write("<mo_selfThinAsymptote>");
    oOut.write("<mo_staVal species=\"Species_1\">0.1019</mo_staVal>");
    oOut.write("</mo_selfThinAsymptote>");
    oOut.write("<mo_selfThinDiamEffect>");
    oOut.write("<mo_stdieVal species=\"Species_1\">0.5391</mo_stdieVal>");
    oOut.write("</mo_selfThinDiamEffect>");
    oOut.write("<mo_selfThinDensityEffect>");
    oOut.write("<mo_stdeeVal species=\"Species_1\">0.00000877</mo_stdeeVal>");
    oOut.write("</mo_selfThinDensityEffect>");
    oOut.write("</mortality>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }
}

