package javawrapper;


import java.io.FileWriter;
import java.io.File;

/**
 * Tests the GrowthBehaviors class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */
public class TestGrowthBehaviors
    extends ModelTestCase {

  /**
   * This test makes sure that lambdas are reset correctly if species are
   * changed.
   */
  public void testDoSetup() {
    try {
      ModelVector p_oData;
      int i;

      //First - set up the way it is
      GUIManager oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.GetTreePopulation().SetSpeciesNames(new String[] {"ACRU", "ACSA"});
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      oGrowth.DoSetup(oManager.GetTreePopulation());

      //Set some lambda values
      Float[] p_fFloat = new Float[] {new Float(0.01), new Float(0.02)};
      Behavior oBeh = oGrowth.GetBehaviorByKey("nci");
      for (i = 0; i < oBeh.GetNumberOfRequiredDataObjects(); i++) {
        ModelData oData = (ModelData) oBeh.GetRequiredData(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("lambda") > -1 && 
            sDescriptor.indexOf("juvenile") == -1) {
          p_oData = (ModelVector) oBeh.GetRequiredData(i);
          oGrowth.SetVectorValues(p_oData, p_fFloat);
          p_fFloat[0] = new Float(0.11);
          p_fFloat[1] = new Float(0.12);
          p_oData = (ModelVector) oBeh.GetRequiredData(i+1);
          oGrowth.SetVectorValues(p_oData, p_fFloat);
          break;
        }
      }
      
      p_fFloat[0] = new Float(0.21);
      p_fFloat[1] = new Float(0.22);
      oBeh = oGrowth.GetBehaviorByKey("Juvenile NCI");
      for (i = 0; i < oBeh.GetNumberOfRequiredDataObjects(); i++) {
        ModelData oData = (ModelData) oBeh.GetRequiredData(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("lambda") > -1 && 
            sDescriptor.indexOf("juvenile") > -1) {
          p_oData = (ModelVector) oBeh.GetRequiredData(i);
          oGrowth.SetVectorValues(p_oData, p_fFloat);
          p_fFloat[0] = new Float(0.31);
          p_fFloat[1] = new Float(0.32);
          p_oData = (ModelVector) oBeh.GetRequiredData(i+1);
          oGrowth.SetVectorValues(p_oData, p_fFloat);
          break;
        }
      }
      
      

      
      
      //********************************************** 
      //Make sure there are ACRU and ACSA lambda neighbors in the whole
      //data list
      boolean bFoundNCIACRU = false,
          bFoundNCIACSA = false;

      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        ModelData oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1 &&
            sDescriptor.indexOf("juvenile") == -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            if (bFoundNCIACSA) {
              fail("Found NCI ACSA more than once.");
            }
            bFoundNCIACSA = true;
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);
      assertTrue(bFoundNCIACSA);

      bFoundNCIACRU = false;
      bFoundNCIACSA = false;

      //Now check the individual behaviors
      oBeh = oGrowth.GetBehaviorByKey("nci");
      if (oBeh == null) {
        fail("Can't find NCI behavior.");
      }
      for (i = 0; i < oBeh.mp_oRequiredData.size(); i++) {
        ModelData oData = (ModelData) oBeh.mp_oRequiredData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1 &&
            sDescriptor.indexOf("juvenile") == -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            if (bFoundNCIACSA) {
              fail("Found NCI ACSA more than once.");
            }
            bFoundNCIACSA = true;
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);
      assertTrue(bFoundNCIACSA);
      
      // ** Repeat with juvenile NCI **
      //Make sure there are ACRU and ACSA lambda neighbors in the whole
      //data list
      bFoundNCIACRU = false;
      bFoundNCIACSA = false;

      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        ModelData oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1 &&
            sDescriptor.indexOf("juvenile") > -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            if (bFoundNCIACSA) {
              fail("Found NCI ACSA more than once.");
            }
            bFoundNCIACSA = true;
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);
      assertTrue(bFoundNCIACSA);

      bFoundNCIACRU = false;
      bFoundNCIACSA = false;

      //Now check the individual behaviors
      oBeh = oGrowth.GetBehaviorByKey("Juvenile NCI");
      if (oBeh == null) {
        fail("Can't find NCI behavior.");
      }
      for (i = 0; i < oBeh.mp_oRequiredData.size(); i++) {
        ModelData oData = (ModelData) oBeh.mp_oRequiredData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1 &&
            sDescriptor.indexOf("juvenile") > -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            if (bFoundNCIACSA) {
              fail("Found NCI ACSA more than once.");
            }
            bFoundNCIACSA = true;
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);
      assertTrue(bFoundNCIACSA);

      //************************
       // Now remove a species
       //************************
      oManager.GetTreePopulation().SetSpeciesNames(new String[] {"ACRU"});
      //Make sure there are ACRU lambda neighbors only in the whole
      //data list
      bFoundNCIACRU = false;
      bFoundNCIACSA = false;

      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        ModelData oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1 &&
            sDescriptor.indexOf("juvenile") == -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
            //Make sure it has the values we set for it
            p_oData = (ModelVector) oData;
            assertEquals(0.01,((Float)p_oData.GetValue().elementAt(0)).floatValue(), 0.001);
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            fail("Found NCI ACSA in all data when it should be gone.");
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);

      bFoundNCIACRU = false;
      bFoundNCIACSA = false;

      //Now check the individual behaviors
      oBeh = oGrowth.GetBehaviorByKey("nci");
      if (oBeh == null) {
        fail("Can't find NCI behavior.");
      }
      for (i = 0; i < oBeh.mp_oRequiredData.size(); i++) {
        ModelData oData = (ModelData) oBeh.mp_oRequiredData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1 &&
            sDescriptor.indexOf("juvenile") == -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            fail("Found NCI ACSA in behavior when it should be gone.");
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);
      
      // ** Repeat with juvenile NCI **
      //Make sure there are ACRU lambda neighbors only in the whole
      //data list
      bFoundNCIACRU = false;
      bFoundNCIACSA = false;

      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        ModelData oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1 &&
            sDescriptor.indexOf("juvenile") > -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
            //Make sure it has the values we set for it
            p_oData = (ModelVector) oData;
            assertEquals(0.21,((Float)p_oData.GetValue().elementAt(0)).floatValue(), 0.001);
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            fail("Found NCI ACSA in all data when it should be gone.");
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);

      bFoundNCIACRU = false;
      bFoundNCIACSA = false;

      //Now check the individual behaviors
      oBeh = oGrowth.GetBehaviorByKey("Juvenile NCI");
      if (oBeh == null) {
        fail("Can't find NCI behavior.");
      }
      for (i = 0; i < oBeh.mp_oRequiredData.size(); i++) {
        ModelData oData = (ModelData) oBeh.mp_oRequiredData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1 &&
            sDescriptor.indexOf("juvenile") > -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            fail("Found NCI ACSA in behavior when it should be gone.");
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);

      //************************
       //Reset our species back to normal
       //************************
      oManager.GetTreePopulation().SetSpeciesNames(new String[] {"ACRU", "ACSA"});
      //Make sure there are ACRU and ACSA lambda neighbors in the whole
      //data list
      bFoundNCIACRU = false;
      bFoundNCIACSA = false;

      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        ModelData oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1 &&
            sDescriptor.indexOf("juvenile") == -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            if (bFoundNCIACSA) {
              fail("Found NCI ACSA more than once.");
            }
            bFoundNCIACSA = true;
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);
      assertTrue(bFoundNCIACSA);

      bFoundNCIACRU = false;
      bFoundNCIACSA = false;

      //Now check the individual behaviors
      oBeh = oGrowth.GetBehaviorByKey("nci");
      if (oBeh == null) {
        fail("Can't find NCI behavior.");
      }
      for (i = 0; i < oBeh.mp_oRequiredData.size(); i++) {
        ModelData oData = (ModelData) oBeh.mp_oRequiredData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1 &&
            sDescriptor.indexOf("juvenile") == -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            if (bFoundNCIACSA) {
              fail("Found NCI ACSA more than once.");
            }
            bFoundNCIACSA = true;
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);
      assertTrue(bFoundNCIACSA);
      
      // ** Repeat with juvenile NCI **
      //Make sure there are ACRU and ACSA lambda neighbors in the whole
      //data list
      bFoundNCIACRU = false;
      bFoundNCIACSA = false;

      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        ModelData oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1 &&
            sDescriptor.indexOf("juvenile") > -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            if (bFoundNCIACSA) {
              fail("Found NCI ACSA more than once.");
            }
            bFoundNCIACSA = true;
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);
      assertTrue(bFoundNCIACSA);

      bFoundNCIACRU = false;
      bFoundNCIACSA = false;

      //Now check the individual behaviors
      oBeh = oGrowth.GetBehaviorByKey("Juvenile NCI");
      if (oBeh == null) {
        fail("Can't find NCI behavior.");
      }
      for (i = 0; i < oBeh.mp_oRequiredData.size(); i++) {
        ModelData oData = (ModelData) oBeh.mp_oRequiredData.elementAt(i);
        String sDescriptor = oData.GetDescriptor().toLowerCase();
        if (sDescriptor.indexOf("nci") > -1 &&
            sDescriptor.indexOf("lambda") > -1 &&
            sDescriptor.indexOf("juvenile") > -1) {
          if (sDescriptor.indexOf("acru") > -1) {
            if (bFoundNCIACRU) {
              fail("Found NCI ACRU more than once.");
            }
            bFoundNCIACRU = true;
          }
          else if (sDescriptor.indexOf("acsa") > -1) {
            if (bFoundNCIACSA) {
              fail("Found NCI ACSA more than once.");
            }
            bFoundNCIACSA = true;
          }
          else {
            fail("Found some NCI that I can't figure out what the hell it is.");
          }
        }
      }

      //Make sure we found everything
      assertTrue(bFoundNCIACRU);
      assertTrue(bFoundNCIACSA);
    }
    catch (ModelException oErr) {
      fail("Caught error with message " + oErr.getMessage());
    }
  }

  /**
   * Tests that the correct parameters are displayed for each behavior.
   */
  public void testFormatDataForDisplay() {
    try {
      GUIManager oManager = new GUIManager(null);
      TreePopulation oPop;
      GrowthBehaviors oGrowth = null;
      String[] p_sExpected;
      Behavior oBeh;

      //Case:  no growth behaviors are enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oPop = oManager.GetTreePopulation();
      oGrowth = oManager.GetGrowthBehaviors();
      assertEquals(null, oGrowth.FormatDataForDisplay(oPop));

      //Case:  only absolute radial growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("absradial");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Mortality Threshold for Suppression",
          "Years Exceeding Threshold Before a Tree is Suppressed",
          "Asymptotic Diameter Growth (A)",
          "Adult Constant Radial Growth in mm/yr",
          "Slope of Diameter Growth Response (S)",
          "Length of Last Suppression Factor",
          "Length of Current Release Factor"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only absolute basal area growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("absba");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Mortality Threshold for Suppression",
          "Years Exceeding Threshold Before a Tree is Suppressed",
          "Asymptotic Diameter Growth (A)",
          "Adult Constant Area Growth in sq. cm/yr",
          "Slope of Diameter Growth Response (S)",
          "Length of Last Suppression Factor",
          "Length of Current Release Factor"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only absolute unlimited growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oPop = oManager.GetTreePopulation();
      oGrowth = oManager.GetGrowthBehaviors();
      oBeh = oGrowth.GetBehaviorByKey("absunlim");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Mortality Threshold for Suppression",
          "Years Exceeding Threshold Before a Tree is Suppressed",
          "Asymptotic Diameter Growth (A)",
          "Slope of Diameter Growth Response (S)",
          "Length of Last Suppression Factor",
          "Length of Current Release Factor"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only relative radial growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("relradial");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Diameter Growth Response (S)",
          "Relative Michaelis-Menton Growth - Diameter Exponent",
          "Adult Constant Radial Growth in mm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only relative basal area growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("relba");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Diameter Growth Response (S)",
          "Relative Michaelis-Menton Growth - Diameter Exponent",
          "Adult Constant Area Growth in sq. cm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only relative unlimited growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("relunlim");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Relative Michaelis-Menton Growth - Diameter Exponent",
          "Slope of Diameter Growth Response (S)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only constant radial growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("constradial");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Adult Constant Radial Growth in mm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only constant basal area growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("constba");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Adult Constant Area Growth in sq. cm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only nci growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("nci");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "NCI Maximum Potential Growth, cm/yr",
          "NCI Maximum Crowding Distance, in meters",
          "NCI Alpha",
          "NCI Beta",
          "NCI Size Sensitivity to NCI (gamma)",
          "NCI Crowding Effect Slope (C)",
          "NCI Crowding Effect Steepness (D)",
          "NCI Neighbor Storm Damage (eta) - Medium (0-1)",
          "NCI Neighbor Storm Damage (eta) - Complete (0-1)",
          "NCI Size Effect Mode, in cm (X0)",
          "NCI Size Effect Variance, in cm (Xb)",
          "NCI Shading Effect Coefficient (m)",
          "NCI Shading Effect Exponent (n)",
          "NCI Damage Effect - Medium Storm Damage (0-1)",
          "NCI Damage Effect - Complete Storm Damage (0-1)",
          "NCI DBH Divisor (q)",
          "Include Snags in NCI Calculations",
          "NCI Minimum Neighbor DBH, in cm",
          "Species 1 NCI Lambda Neighbors",
          "Species 2 NCI Lambda Neighbors"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);
      
      //Case:  only nci juvenile growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("Juvenile NCI");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 2, oPop));
      p_sExpected = new String[] {
          "Juvenile NCI Maximum Potential Growth, cm/yr",
          "Juvenile NCI Maximum Crowding Distance, in meters",
          "Juvenile NCI Minimum Neighbor Diam10, in cm",
          "Juvenile NCI Alpha",
          "Juvenile NCI Beta",
          "Juvenile NCI Crowding Effect Slope (C)",
          "Juvenile NCI Crowding Effect Steepness (D)",
          "Juvenile NCI Size Effect \"a\"",
          "Juvenile NCI Size Effect \"b\"",
          "Juvenile NCI Diam10 Divisor (q)",
          "Juvenile NCI - Include Snags in NCI Calculations",
          "Species 1 Juvenile NCI Lambda Neighbors",
          "Species 2 Juvenile NCI Lambda Neighbors"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only ba nci growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("nciba");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "NCI Maximum Potential Growth, cm/yr",
          "NCI Maximum Crowding Distance, in meters",
          "NCI Size Sensitivity to NCI (gamma)",
          "NCI Crowding Effect Slope (C)",
          "NCI Crowding Effect Steepness (D)",
          "NCI Size Effect Mode, in cm (X0)",
          "NCI Size Effect Variance, in cm (Xb)",
          "Basal Area NCI - BA Divisor",
          "Basal Area NCI - Use Only Larger Neighbors",
          "NCI Minimum Neighbor DBH, in cm"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only logistic growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("logistic growth");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Logistic - Asymptotic Diam Growth - Full Light in mm/yr (a)",
          "Logistic - Diam Shape Param 1 (b)",
          "Logistic - Diam Shape Param 2 (c)",
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only simple linear growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("linear");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Simple Linear - Diam Intercept in mm/yr (a)",
          "Simple Linear - Diam Slope (b)"
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only linear bi-level growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("linear bi-level");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Linear Bi-Level - Threshold for High-Light Growth (0 - 100)",
          "Linear Bi-Level - Intercept for High-Light Growth (a)",
          "Linear Bi-Level - Slope for Low-Light Growth (b)",
          "Linear Bi-Level - Intercept for Low-Light Growth (a)",
          "Linear Bi-Level - Slope for High-Light Growth (b)"
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only size-dependent logistic growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("size dependent logistic growth");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Size Dep. Logistic - Diam Intercept (a)",
          "Size Dep. Logistic - Diam Slope (b)",
          "Size Dep. Logistic - Diam Shape Param 1 (c)",
          "Size Dep. Logistic - Diam Shape Param 2 (d)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only lognormal growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("lognormal growth");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Lognormal - Diam Growth Increment at Diam 36, in mm/yr (a)",
          "Lognormal - Diam Shape Parameter (b)",
          "Lognormal - Diam Effect of Shade (c)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only shaded linear growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("shaded linear");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Shaded Linear - Diam Intercept in mm/yr (a)",
          "Shaded Linear - Diam Slope (b)",
          "Shaded Linear - Diam Shade Exponent (c)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only float-resource relative growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("float resource");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Diameter Growth Response (S)",
          "Float resource - Influence of Resource (C)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only Puerto Rico storm bi-level growth, auto-diam, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("pr storm bi-level");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "PR Storm Bi-Level - Threshold for High-Light Growth (0 - 100)",
          "PR Storm Bi-Level - High-Light \"a\"",
          "PR Storm Bi-Level - High-Light \"b\"",
          "PR Storm Bi-Level - Slope for Low-Light Growth (b)",
          "PR Storm Bi-Level - Intercept for Low-Light Growth (a)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);


      //Case:  only absolute radial growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("absradial diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Mortality Threshold for Suppression",
          "Years Exceeding Threshold Before a Tree is Suppressed",
          "Asymptotic Diameter Growth (A)",
          "Adult Constant Radial Growth in mm/yr",
          "Slope of Diameter Growth Response (S)",
          "Length of Last Suppression Factor",
          "Length of Current Release Factor"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only absolute basal area growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("absba diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Mortality Threshold for Suppression",
          "Years Exceeding Threshold Before a Tree is Suppressed",
          "Asymptotic Diameter Growth (A)",
          "Adult Constant Area Growth in sq. cm/yr",
          "Slope of Diameter Growth Response (S)",
          "Length of Last Suppression Factor",
          "Length of Current Release Factor"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only absolute unlimited growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("absunlim diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Mortality Threshold for Suppression",
          "Years Exceeding Threshold Before a Tree is Suppressed",
          "Asymptotic Diameter Growth (A)",
          "Slope of Diameter Growth Response (S)",
          "Length of Last Suppression Factor",
          "Length of Current Release Factor"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only relative radial growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("relradial diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Diameter Growth Response (S)",
          "Relative Michaelis-Menton Growth - Diameter Exponent",
          "Adult Constant Radial Growth in mm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only relative basal area growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("relba diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Diameter Growth Response (S)",
          "Relative Michaelis-Menton Growth - Diameter Exponent",
          "Adult Constant Area Growth in sq. cm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only relative unlimited growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("relunlim diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Relative Michaelis-Menton Growth - Diameter Exponent",
          "Slope of Diameter Growth Response (S)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only constant radial growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("constradial diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Adult Constant Radial Growth in mm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only constant basal area growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("constba diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Adult Constant Area Growth in sq. cm/yr"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only nci growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("nci diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "NCI Maximum Potential Growth, cm/yr",
          "NCI Maximum Crowding Distance, in meters",
          "NCI Alpha",
          "NCI Beta",
          "NCI Size Sensitivity to NCI (gamma)",
          "NCI Crowding Effect Slope (C)",
          "NCI Crowding Effect Steepness (D)",
          "NCI Neighbor Storm Damage (eta) - Medium (0-1)",
          "NCI Neighbor Storm Damage (eta) - Complete (0-1)",
          "NCI Size Effect Mode, in cm (X0)",
          "NCI Size Effect Variance, in cm (Xb)",
          "NCI Shading Effect Coefficient (m)",
          "NCI Shading Effect Exponent (n)",
          "NCI Damage Effect - Medium Storm Damage (0-1)",
          "NCI Damage Effect - Complete Storm Damage (0-1)",
          "NCI DBH Divisor (q)",
          "Include Snags in NCI Calculations",
          "NCI Minimum Neighbor DBH, in cm",
          "Species 1 NCI Lambda Neighbors",
          "Species 2 NCI Lambda Neighbors"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);
      
      //Case:  only nci juvenile growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("Juvenile NCI diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 2, oPop));
      p_sExpected = new String[] {
          "Juvenile NCI Maximum Potential Growth, cm/yr",
          "Juvenile NCI Maximum Crowding Distance, in meters",
          "Juvenile NCI Minimum Neighbor Diam10, in cm",
          "Juvenile NCI Alpha",
          "Juvenile NCI Beta",
          "Juvenile NCI Crowding Effect Slope (C)",
          "Juvenile NCI Crowding Effect Steepness (D)",
          "Juvenile NCI Size Effect \"a\"",
          "Juvenile NCI Size Effect \"b\"",
          "Juvenile NCI Diam10 Divisor (q)",
          "Juvenile NCI - Include Snags in NCI Calculations",
          "Species 1 Juvenile NCI Lambda Neighbors",
          "Species 2 Juvenile NCI Lambda Neighbors"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only ba nci growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("nciba diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "NCI Maximum Potential Growth, cm/yr",
          "NCI Maximum Crowding Distance, in meters",
          "NCI Size Sensitivity to NCI (gamma)",
          "NCI Crowding Effect Slope (C)",
          "NCI Crowding Effect Steepness (D)",
          "NCI Size Effect Mode, in cm (X0)",
          "NCI Size Effect Variance, in cm (Xb)",
          "Basal Area NCI - BA Divisor",
          "Basal Area NCI - Use Only Larger Neighbors",
          "NCI Minimum Neighbor DBH, in cm"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only logistic growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("logistic growth diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Logistic - Asymptotic Diam Growth - Full Light in mm/yr (a)",
          "Logistic - Diam Shape Param 1 (b)",
          "Logistic - Diam Shape Param 2 (c)",
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only linear bi-level growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("linear bi-level diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Linear Bi-Level - Threshold for High-Light Growth (0 - 100)",
          "Linear Bi-Level - Intercept for High-Light Growth (a)",
          "Linear Bi-Level - Slope for Low-Light Growth (b)",
          "Linear Bi-Level - Intercept for Low-Light Growth (a)",
          "Linear Bi-Level - Slope for High-Light Growth (b)"
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only Simple Linear growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("linear diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Simple Linear - Diam Intercept in mm/yr (a)",
          "Simple Linear - Diam Slope (b)"
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only size-dependent logistic growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey(
          "size dependent logistic growth diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Size Dep. Logistic - Diam Intercept (a)",
          "Size Dep. Logistic - Diam Slope (b)",
          "Size Dep. Logistic - Diam Shape Param 1 (c)",
          "Size Dep. Logistic - Diam Shape Param 2 (d)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only lognormal growth, Diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("lognormal growth diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Lognormal - Diam Growth Increment at Diam 36, in mm/yr (a)",
          "Lognormal - Diam Shape Parameter (b)",
          "Lognormal - Diam Effect of Shade (c)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only Shaded Linear growth, Diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("shaded linear diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Shaded Linear - Diam Intercept in mm/yr (a)",
          "Shaded Linear - Diam Slope (b)",
          "Shaded Linear - Diam Shade Exponent (c)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only float-resource relative growth, diam only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("float resource diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Diameter Growth Response (S)",
          "Float resource - Influence of Resource (C)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only height incrementer is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("height incrementer");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      assertEquals(null, oGrowth.FormatDataForDisplay(oPop));

      //Case:  only logistic growth, height only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("logistic growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Logistic - Asymptotic Height Growth - Full Light in cm/yr (a)",
          "Logistic - Height Shape Param 1 (b)",
          "Logistic - Height Shape Param 2 (c)",
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only Simple Linear growth, height only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("linear height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Simple Linear - Height Intercept in cm/yr (a)",
          "Simple Linear - Height Slope (b)"
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);
      
      //Case:  only Michaelis Menton with negative growth - height only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("Michaelis Menton with negative growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Michaelis-Menton Neg Growth - Alpha",
          "Michaelis-Menton Neg Growth - Beta",
          "Michaelis-Menton Neg Growth - Gamma",
          "Michaelis-Menton Neg Growth - Phi",
          "Michaelis-Menton Neg Growth - Growth Standard Deviation",
          "Michaelis-Menton Neg Growth - Autocorrelation Prob (0-1)"
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);
      
      //Case:  only Michaelis Menton with photoinhibition - height only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("Michaelis Menton with photoinhibition height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Michaelis-Menton with Photoinhibition - Alpha",
          "Michaelis-Menton with Photoinhibition - Beta",
          "Michaelis-Menton with Photoinhibition - D",
          "Michaelis-Menton with Photoinhibition - Phi"
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only size-dependent logistic growth, height only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey(
          "size dependent logistic growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Size Dep. Logistic - Height Intercept (a)",
          "Size Dep. Logistic - Height Slope (b)",
          "Size Dep. Logistic - Height Shape Param 1 (c)",
          "Size Dep. Logistic - Height Shape Param 2 (d)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only lognormal growth, height only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("lognormal growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Lognormal - Height Growth Increment at Diam 36, in cm/yr (a)",
          "Lognormal - Height Shape Parameter (b)",
          "Lognormal - Height Effect of Shade (c)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only Shaded Linear growth, height only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("shaded linear height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Shaded Linear - Height Intercept in cm/yr (a)",
          "Shaded Linear - Height Slope (b)",
          "Shaded Linear - Height Shade Exponent (c)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only Shaded Linear growth, height only, is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Lognormal Bi-Level - X0 for Low-Light Growth",
          "Lognormal Bi-Level - Xb for Low-Light Growth",
          "Lognormal Bi-Level - X0 for High-Light Growth",
          "Lognormal Bi-Level - Xb for High-Light Growth",
          "Lognormal Bi-Level - Max Growth in High Light (m)",
          "Lognormal Bi-Level - Threshold for High-Light Growth (0 - 100)",
          "Lognormal Bi-Level - Max Growth in Low Light (m)"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only PR semi-stochastic diameter growth is enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("pr semi-stochastic diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "PR - Height Threshold for Stochastic Growth (m)",
          "PR - \"a\" Parameter for Deterministic Growth",
          "PR - \"b\" Parameter for Deterministic Growth",
          "PR - Mean DBH (cm) for Stochastic Growth",
          "PR - DBH Standard Deviation for Stochastic Growth"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only browsed relative growth, diam with auto height, is enabled
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("browsed relative growth");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Diameter Growth Response (S)",
          "Relative Michaelis-Menton Growth - Diameter Exponent",
          "Browsed Asymptotic Diameter Growth (A)",
          "Browsed Slope of Growth Response (S)",
          "Browsed Diameter Exponent"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only browsed relative growth, diam only, is enabled
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("browsed relative growth diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Asymptotic Diameter Growth (A)",
          "Slope of Diameter Growth Response (S)",
          "Relative Michaelis-Menton Growth - Diameter Exponent",
          "Browsed Asymptotic Diameter Growth (A)",
          "Browsed Slope of Growth Response (S)",
          "Browsed Diameter Exponent"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);
      
      //Case:  only power growth, diam only, is enabled
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("Power growth - height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Power Height Growth - n",
          "Power Height Growth - Exp"};
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);
      
      //Case: only lagged post harvest growth, diam with auto height, is enabled 
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("lagged post harvest growth");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Post Harvest Growth - Max Growth Constant",
          "Post Harvest Growth - DBH Growth Effect",
          "Post Harvest Growth - NCI Constant ",
          "Post Harvest Growth - DBH NCI Effect",
          "Post Harvest Growth - Time Since Harvest Rate Param",
          "Post Harvest Growth - NCI Distance (m)"
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      //Case: only Weibull climate growth, diam with auto height, is enabled
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("Weibull climate");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Weibull Climate Growth - Adult Competition Effect \"C\"",
          "Weibull Climate Growth - Juvenile Competition Effect \"C\"",
          "Weibull Climate Growth - Adult Competition Effect \"D\"",
          "Weibull Climate Growth - Juvenile Competition Effect \"D\"",
          "Weibull Climate Growth - Adult Competition Gamma",
          "Weibull Climate Growth - Juvenile Competition Gamma",
          "Weibull Climate Growth - Max Neighbor Search Radius (m)",
          "Weibull Climate Growth - Minimum Neighbor DBH (cm)",
          "Weibull Climate Growth - Size Effect X0",
          "Weibull Climate Growth - Size Effect Xb",
          "Weibull Climate Growth - Size Effect Minimum DBH",
          "Weibull Climate Growth - Adult Precip Effect \"A\"",
          "Weibull Climate Growth - Juvenile Precip Effect \"A\"",
          "Weibull Climate Growth - Adult Precip Effect \"B\"",
          "Weibull Climate Growth - Juvenile Precip Effect \"B\"",
          "Weibull Climate Growth - Adult Precip Effect \"C\"",
          "Weibull Climate Growth - Juvenile Precip Effect \"C\"",
          "Weibull Climate Growth - Adult Temp Effect \"A\"",
          "Weibull Climate Growth - Juvenile Temp Effect \"A\"",
          "Weibull Climate Growth - Adult Temp Effect \"B\"",
          "Weibull Climate Growth - Juvenile Temp Effect \"B\"",
          "Weibull Climate Growth - Adult Temp Effect \"C\"",
          "Weibull Climate Growth - Juvenile Temp Effect \"C\"",
          "Weibull Climate Growth - Max Potential Growth (cm/yr)",
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);
      
    //Case: only Weibull climate growth, diam only, is enabled
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("Weibull climate diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Weibull Climate Growth - Adult Competition Effect \"C\"",
          "Weibull Climate Growth - Juvenile Competition Effect \"C\"",
          "Weibull Climate Growth - Adult Competition Effect \"D\"",
          "Weibull Climate Growth - Juvenile Competition Effect \"D\"",
          "Weibull Climate Growth - Adult Competition Gamma",
          "Weibull Climate Growth - Juvenile Competition Gamma",
          "Weibull Climate Growth - Max Neighbor Search Radius (m)",
          "Weibull Climate Growth - Minimum Neighbor DBH (cm)",
          "Weibull Climate Growth - Size Effect X0",
          "Weibull Climate Growth - Size Effect Xb",
          "Weibull Climate Growth - Size Effect Minimum DBH",
          "Weibull Climate Growth - Adult Precip Effect \"A\"",
          "Weibull Climate Growth - Juvenile Precip Effect \"A\"",
          "Weibull Climate Growth - Adult Precip Effect \"B\"",
          "Weibull Climate Growth - Juvenile Precip Effect \"B\"",
          "Weibull Climate Growth - Adult Precip Effect \"C\"",
          "Weibull Climate Growth - Juvenile Precip Effect \"C\"",
          "Weibull Climate Growth - Adult Temp Effect \"A\"",
          "Weibull Climate Growth - Juvenile Temp Effect \"A\"",
          "Weibull Climate Growth - Adult Temp Effect \"B\"",
          "Weibull Climate Growth - Juvenile Temp Effect \"B\"",
          "Weibull Climate Growth - Adult Temp Effect \"C\"",
          "Weibull Climate Growth - Juvenile Temp Effect \"C\"",
          "Weibull Climate Growth - Max Potential Growth (cm/yr)",
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);
      
      //Case: only Weibull climate quadrat growth, diam with auto height, is enabled
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("Weibull climate quadrat growth");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Weib Clim Quad Growth - Competition Effect \"C\"",
          "Weib Clim Quad Growth - Competition Effect \"D\"",
          "Weib Clim Quad Growth - Minimum Neighbor DBH (cm)",
          "Weib Clim Quad Growth - Precip Effect \"A\"",
          "Weib Clim Quad Growth - Precip Effect \"B\"",
          "Weib Clim Quad Growth - Precip Effect \"C\"",
          "Weib Clim Quad Growth - Temp Effect \"A\"",
          "Weib Clim Quad Growth - Temp Effect \"B\"",
          "Weib Clim Quad Growth - Temp Effect \"C\"",
          "Weib Clim Quad Growth - Max Potential Growth (cm/yr)",
          "Weib Clim Quad Growth - Max Neighbor Search Radius (m)"
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);
          
      //Case: only Weibull climate quadrat growth, diam only, is enabled
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oGrowth.GetBehaviorByKey("Weibull climate quadrat growth diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Weib Clim Quad Growth - Competition Effect \"C\"",
          "Weib Clim Quad Growth - Competition Effect \"D\"",
          "Weib Clim Quad Growth - Minimum Neighbor DBH (cm)",
          "Weib Clim Quad Growth - Precip Effect \"A\"",
          "Weib Clim Quad Growth - Precip Effect \"B\"",
          "Weib Clim Quad Growth - Precip Effect \"C\"",
          "Weib Clim Quad Growth - Temp Effect \"A\"",
          "Weib Clim Quad Growth - Temp Effect \"B\"",
          "Weib Clim Quad Growth - Temp Effect \"C\"",
          "Weib Clim Quad Growth - Max Potential Growth (cm/yr)",
          "Weib Clim Quad Growth - Max Neighbor Search Radius (m)"
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);
      
      //Case:  all are enabled.
      oManager.InputXMLParameterFile(WriteNoGrowthXMLFile1());
      oGrowth = oManager.GetGrowthBehaviors();
      oPop = oManager.GetTreePopulation();
      int i;
      for (i = 0; i < oGrowth.mp_oChildBehaviors.length; i++) {
        oGrowth.mp_oChildBehaviors[i].m_bIsEnabled = true;
        if (oGrowth.mp_oChildBehaviors[i].m_sKey.indexOf("Juvenile NCI") > -1 )
          oGrowth.mp_oChildBehaviors[i].AddSpeciesTypeCombo(new SpeciesTypeCombo(
              0, 2, oPop));
        else
          oGrowth.mp_oChildBehaviors[i].AddSpeciesTypeCombo(new SpeciesTypeCombo(
            0, 3, oPop));
      }
      p_sExpected = new String[] {
          "NCI Maximum Potential Growth, cm/yr",
          "NCI Maximum Crowding Distance, in meters",
          "NCI Alpha",
          "NCI Beta",
          "NCI Size Sensitivity to NCI (gamma)",
          "NCI Crowding Effect Slope (C)",
          "NCI Crowding Effect Steepness (D)",
          "NCI Neighbor Storm Damage (eta) - Medium (0-1)",
          "NCI Neighbor Storm Damage (eta) - Complete (0-1)",
          "NCI Size Effect Mode, in cm (X0)",
          "NCI Size Effect Variance, in cm (Xb)",
          "NCI Shading Effect Coefficient (m)",
          "NCI Shading Effect Exponent (n)",
          "NCI Damage Effect - Medium Storm Damage (0-1)",
          "NCI Damage Effect - Complete Storm Damage (0-1)",
          "NCI DBH Divisor (q)",
          "Include Snags in NCI Calculations",
          "NCI Minimum Neighbor DBH, in cm",
          "Species 1 NCI Lambda Neighbors",
          "Species 2 NCI Lambda Neighbors",
          "Relative Michaelis-Menton Growth - Diameter Exponent",
          "Relative Michaelis-Menton Growth - Height Exponent",
          "Slope of Diameter Growth Response (S)",
          "Slope of Height Growth Response (S)",
          "Mortality Threshold for Suppression",
          "Years Exceeding Threshold Before a Tree is Suppressed",
          "Asymptotic Diameter Growth (A)",
          "Asymptotic Height Growth (A)",
          "Adult Constant Radial Growth in mm/yr",
          "Adult Constant Area Growth in sq. cm/yr",
          "Length of Last Suppression Factor",
          "Length of Current Release Factor",
          "Logistic - Asymptotic Diam Growth - Full Light in mm/yr (a)",
          "Logistic - Asymptotic Height Growth - Full Light in cm/yr (a)",
          "Logistic - Diam Shape Param 1 (b)",
          "Logistic - Height Shape Param 1 (b)",
          "Logistic - Diam Shape Param 2 (c)",
          "Logistic - Height Shape Param 2 (c)",
          "Simple Linear - Diam Intercept in mm/yr (a)",
          "Simple Linear - Height Intercept in cm/yr (a)",
          "Simple Linear - Diam Slope (b)",
          "Simple Linear - Height Slope (b)",
          "Shaded Linear - Diam Intercept in mm/yr (a)",
          "Shaded Linear - Height Intercept in cm/yr (a)",
          "Shaded Linear - Diam Slope (b)",
          "Shaded Linear - Height Slope (b)",
          "Shaded Linear - Diam Shade Exponent (c)",
          "Shaded Linear - Height Shade Exponent (c)",
          "Size Dep. Logistic - Diam Intercept (a)",
          "Size Dep. Logistic - Height Intercept (a)",
          "Size Dep. Logistic - Diam Slope (b)",
          "Size Dep. Logistic - Height Slope (b)",
          "Size Dep. Logistic - Diam Shape Param 1 (c)",
          "Size Dep. Logistic - Height Shape Param 1 (c)",
          "Size Dep. Logistic - Diam Shape Param 2 (d)",
          "Size Dep. Logistic - Height Shape Param 2 (d)",
          "Lognormal - Diam Growth Increment at Diam 36, in mm/yr (a)",
          "Lognormal - Height Growth Increment at Diam 36, in cm/yr (a)",
          "Lognormal - Diam Shape Parameter (b)",
          "Lognormal - Height Shape Parameter (b)",
          "Lognormal - Diam Effect of Shade (c)",
          "Lognormal - Height Effect of Shade (c)",
          "Float resource - Influence of Resource (C)",
          "Linear Bi-Level - Threshold for High-Light Growth (0 - 100)",
          "Linear Bi-Level - Intercept for High-Light Growth (a)",
          "Linear Bi-Level - Slope for Low-Light Growth (b)",
          "Linear Bi-Level - Intercept for Low-Light Growth (a)",
          "Linear Bi-Level - Slope for High-Light Growth (b)",
          "Lognormal Bi-Level - X0 for Low-Light Growth",
          "Lognormal Bi-Level - Xb for Low-Light Growth",
          "Lognormal Bi-Level - X0 for High-Light Growth",
          "Lognormal Bi-Level - Xb for High-Light Growth",
          "Lognormal Bi-Level - Max Growth in High Light (m)",
          "Lognormal Bi-Level - Threshold for High-Light Growth (0 - 100)",
          "Lognormal Bi-Level - Max Growth in Low Light (m)",
          "PR - Height Threshold for Stochastic Growth (m)",
          "PR - \"a\" Parameter for Deterministic Growth",
          "PR - \"b\" Parameter for Deterministic Growth",
          "PR - Mean DBH (cm) for Stochastic Growth",
          "PR - DBH Standard Deviation for Stochastic Growth",
          "PR Storm Bi-Level - Threshold for High-Light Growth (0 - 100)",
          "PR Storm Bi-Level - High-Light \"a\"",
          "PR Storm Bi-Level - High-Light \"b\"",
          "PR Storm Bi-Level - Slope for Low-Light Growth (b)",
          "PR Storm Bi-Level - Intercept for Low-Light Growth (a)",
          "Michaelis-Menton Neg Growth - Alpha",
          "Michaelis-Menton Neg Growth - Beta",
          "Michaelis-Menton Neg Growth - Gamma",
          "Michaelis-Menton Neg Growth - Phi",
          "Michaelis-Menton Neg Growth - Growth Standard Deviation",
          "Michaelis-Menton Neg Growth - Autocorrelation Prob (0-1)",
          "Michaelis-Menton with Photoinhibition - Alpha",
          "Michaelis-Menton with Photoinhibition - Beta",
          "Michaelis-Menton with Photoinhibition - D",
          "Michaelis-Menton with Photoinhibition - Phi",
          "Browsed Asymptotic Diameter Growth (A)",
          "Browsed Slope of Growth Response (S)",
          "Browsed Diameter Exponent",
          "Power Height Growth - n",
          "Power Height Growth - Exp",
          "Basal Area NCI - BA Divisor",
          "Basal Area NCI - Use Only Larger Neighbors",
          "Post Harvest Growth - Max Growth Constant",
          "Post Harvest Growth - DBH Growth Effect",
          "Post Harvest Growth - NCI Constant ",
          "Post Harvest Growth - DBH NCI Effect",
          "Post Harvest Growth - Time Since Harvest Rate Param",
          "Post Harvest Growth - NCI Distance (m)",
          "Juvenile NCI Maximum Potential Growth, cm/yr",
          "Juvenile NCI Maximum Crowding Distance, in meters",
          "Juvenile NCI Minimum Neighbor Diam10, in cm",
          "Juvenile NCI Alpha",
          "Juvenile NCI Beta",
          "Juvenile NCI Crowding Effect Slope (C)",
          "Juvenile NCI Crowding Effect Steepness (D)",
          "Juvenile NCI Size Effect \"a\"",
          "Juvenile NCI Size Effect \"b\"",
          "Juvenile NCI Diam10 Divisor (q)",
          "Juvenile NCI - Include Snags in NCI Calculations",
          "Species 1 Juvenile NCI Lambda Neighbors",
          "Species 2 Juvenile NCI Lambda Neighbors",
          "Weibull Climate Growth - Adult Competition Effect \"C\"",
          "Weibull Climate Growth - Juvenile Competition Effect \"C\"",
          "Weibull Climate Growth - Adult Competition Effect \"D\"",
          "Weibull Climate Growth - Juvenile Competition Effect \"D\"",
          "Weibull Climate Growth - Adult Competition Gamma",
          "Weibull Climate Growth - Juvenile Competition Gamma",
          "Weibull Climate Growth - Max Neighbor Search Radius (m)",
          "Weibull Climate Growth - Minimum Neighbor DBH (cm)",
          "Weibull Climate Growth - Size Effect X0",
          "Weibull Climate Growth - Size Effect Xb",
          "Weibull Climate Growth - Size Effect Minimum DBH",
          "Weibull Climate Growth - Adult Precip Effect \"A\"",
          "Weibull Climate Growth - Juvenile Precip Effect \"A\"",
          "Weibull Climate Growth - Adult Precip Effect \"B\"",
          "Weibull Climate Growth - Juvenile Precip Effect \"B\"",
          "Weibull Climate Growth - Adult Precip Effect \"C\"",
          "Weibull Climate Growth - Juvenile Precip Effect \"C\"",
          "Weibull Climate Growth - Adult Temp Effect \"A\"",
          "Weibull Climate Growth - Juvenile Temp Effect \"A\"",
          "Weibull Climate Growth - Adult Temp Effect \"B\"",
          "Weibull Climate Growth - Juvenile Temp Effect \"B\"",
          "Weibull Climate Growth - Adult Temp Effect \"C\"",
          "Weibull Climate Growth - Juvenile Temp Effect \"C\"",
          "Weibull Climate Growth - Max Potential Growth (cm/yr)",
          "Weib Clim Quad Growth - Competition Effect \"C\"",
          "Weib Clim Quad Growth - Competition Effect \"D\"",
          "Weib Clim Quad Growth - Minimum Neighbor DBH (cm)",
          "Weib Clim Quad Growth - Precip Effect \"A\"",
          "Weib Clim Quad Growth - Precip Effect \"B\"",
          "Weib Clim Quad Growth - Precip Effect \"C\"",
          "Weib Clim Quad Growth - Temp Effect \"A\"",
          "Weib Clim Quad Growth - Temp Effect \"B\"",
          "Weib Clim Quad Growth - Temp Effect \"C\"",
          "Weib Clim Quad Growth - Max Potential Growth (cm/yr)",
          "Weib Clim Quad Growth - Max Neighbor Search Radius (m)"
      };
      TestTable(oGrowth.FormatDataForDisplay(oPop), p_sExpected);

      System.out.println("FormatDataForDisplay succeeded for growth.");
    }
    catch (ModelException oErr) {
      fail("Caught error with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
       fail("Caught IOException.  Message: " + oE.getMessage());
     }

  }

  /**
   * Makes sure that nci lambda values are correctly read from parameter
   * file.
   */
  public void testReadParFile() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      ModelData oData;
      ModelVector oVector;
      int i, iIndex = 0;

      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1 &&
            oData.GetDescriptor().toLowerCase().indexOf("juvenile") == -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 0.6640108, 0.01);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 0.71, 0.01);
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex + 1);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 0.00442797, 0.01);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 0.12, 0.01);
      
      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1 &&
            oData.GetDescriptor().toLowerCase().indexOf("juvenile") > -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 0.1, 0.01);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 0.2, 0.01);
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex + 1);
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 0.3, 0.01);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 0.4, 0.01);

      assertEquals(((Number)oGrowth.mp_fWeibClimMaxCrowdingRadius.GetValue().elementAt(0)).floatValue(),
          (float) 10, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimMaxCrowdingRadius.GetValue().elementAt(1)).floatValue(),
          (float) 11, 0.01);
      
      assertEquals(((Number)oGrowth.mp_fWeibClimMaxRG.GetValue().elementAt(0)).floatValue(),
          (float) 0.2574, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimMaxRG.GetValue().elementAt(1)).floatValue(),
          (float) 10, 0.01);
      
      assertEquals(((Number)oGrowth.mp_fWeibClimSizeX0.GetValue().elementAt(0)).floatValue(),
          (float) 297.615, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimSizeX0.GetValue().elementAt(1)).floatValue(),
          (float) 18.07, 0.01);
      
      assertEquals(((Number)oGrowth.mp_fWeibClimSizeXb.GetValue().elementAt(0)).floatValue(),
          (float) 4.065, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimSizeXb.GetValue().elementAt(1)).floatValue(),
          (float) 0.902, 0.01);
      
      assertEquals(((Number)oGrowth.mp_fWeibClimAdultCompC.GetValue().elementAt(0)).floatValue(),
          (float) 8.261, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimAdultCompC.GetValue().elementAt(1)).floatValue(),
          (float) 0, 0.01);
      
      assertEquals(((Number)oGrowth.mp_fWeibClimAdultCompD.GetValue().elementAt(0)).floatValue(),
          (float) 1.001, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimAdultCompD.GetValue().elementAt(1)).floatValue(),
          (float) 2.914, 0.01);
      
      assertEquals(((Number)oGrowth.mp_fWeibClimAdultGamma.GetValue().elementAt(0)).floatValue(),
          (float) -1.208, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimAdultGamma.GetValue().elementAt(1)).floatValue(),
          (float) 1.976, 0.01);
      
      assertEquals(((Number)oGrowth.mp_fWeibClimJuvenileCompC.GetValue().elementAt(0)).floatValue(),
          (float) 15.16225, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimJuvenileCompC.GetValue().elementAt(1)).floatValue(),
          (float) 10.67038, 0.01);
      
      assertEquals(((Number)oGrowth.mp_fWeibClimJuvenileCompD.GetValue().elementAt(0)).floatValue(),
          (float) 0.9, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimJuvenileCompD.GetValue().elementAt(1)).floatValue(),
          (float) 1.1, 0.01);
      
      assertEquals(((Number)oGrowth.mp_fWeibClimJuvenileGamma.GetValue().elementAt(0)).floatValue(),
          (float) -0.06, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimJuvenileGamma.GetValue().elementAt(1)).floatValue(),
          (float) -0.4, 0.01);
      
      assertEquals(((Number)oGrowth.mp_fWeibClimAdultTempA.GetValue().elementAt(0)).floatValue(),
          (float) 8.8, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimAdultTempA.GetValue().elementAt(1)).floatValue(),
          (float) 8.9, 0.01);
      
      assertEquals(((Number)oGrowth.mp_fWeibClimAdultTempB.GetValue().elementAt(0)).floatValue(),
          (float) 1.2, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimAdultTempB.GetValue().elementAt(1)).floatValue(),
          (float) 1.47, 0.01);
      
      assertEquals(((Number)oGrowth.mp_fWeibClimAdultTempC.GetValue().elementAt(0)).floatValue(),
          (float) 13.2, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimAdultTempC.GetValue().elementAt(1)).floatValue(),
          (float) 12.4, 0.01);
      
      assertEquals(((Number)oGrowth.mp_fWeibClimAdultPrecipA.GetValue().elementAt(0)).floatValue(),
          (float) 420.2, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimAdultPrecipA.GetValue().elementAt(1)).floatValue(),
          (float) 3.7, 0.01);
      
      assertEquals(((Number)oGrowth.mp_fWeibClimAdultPrecipB.GetValue().elementAt(0)).floatValue(),
          (float) 0.4, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimAdultPrecipB.GetValue().elementAt(1)).floatValue(),
          (float) 0.07, 0.01);
      
      assertEquals(((Number)oGrowth.mp_fWeibClimAdultPrecipC.GetValue().elementAt(0)).floatValue(),
          (float) 0.03, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimAdultPrecipC.GetValue().elementAt(1)).floatValue(),
          (float) 1048.7, 0.01);
      
      assertEquals(((Number)oGrowth.mp_fWeibClimJuvenilePrecipA.GetValue().elementAt(0)).floatValue(),
          (float) 1296.2, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimJuvenilePrecipA.GetValue().elementAt(1)).floatValue(),
          (float) 3306.1, 0.01);
      
      assertEquals(((Number)oGrowth.mp_fWeibClimJuvenilePrecipB.GetValue().elementAt(0)).floatValue(),
          (float) 0.8, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimJuvenilePrecipB.GetValue().elementAt(1)).floatValue(),
          (float) 6.6, 0.01);
      
      assertEquals(((Number)oGrowth.mp_fWeibClimJuvenilePrecipC.GetValue().elementAt(0)).floatValue(),
          (float) 1032.2, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimJuvenilePrecipC.GetValue().elementAt(1)).floatValue(),
          (float) 3835.9, 0.01);
      
      assertEquals(((Number)oGrowth.mp_fWeibClimJuvenileTempA.GetValue().elementAt(0)).floatValue(),
          (float) 6.8, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimJuvenileTempA.GetValue().elementAt(1)).floatValue(),
          (float) 139.5, 0.01);
      
      assertEquals(((Number)oGrowth.mp_fWeibClimJuvenileTempB.GetValue().elementAt(0)).floatValue(),
          (float) 0.3, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimJuvenileTempB.GetValue().elementAt(1)).floatValue(),
          (float) 0, 0.01);
      
      assertEquals(((Number)oGrowth.mp_fWeibClimJuvenileTempC.GetValue().elementAt(0)).floatValue(),
          (float) 14.2, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimJuvenileTempC.GetValue().elementAt(1)).floatValue(),
          (float) 28.6, 0.01);
      
      assertEquals(((Number)oGrowth.mp_fWeibClimMinimumNeighborDBH.GetValue().elementAt(0)).floatValue(),
          (float) 5, 0.01);
      assertEquals(((Number)oGrowth.mp_fWeibClimMinimumNeighborDBH.GetValue().elementAt(1)).floatValue(),
          (float) 6, 0.01);
      
      
      
      new File(sFileName).delete();

    }
    catch (ModelException oErr) {
      fail("Growth validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Makes sure that nci lambda values are managed correctly when species
   * change.
   */
  public void testChangeOfSpecies() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      ModelData oData;
      ModelVector oVector;
      int i, iIndex = 0;

      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1 &&
            oData.GetDescriptor().toLowerCase().indexOf("juvenile") == -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 1"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 0.6640108, 0.01);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 0.71, 0.01);

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex + 1);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 2"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 0.00442797, 0.01);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 0.12, 0.01);
      
      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1 &&
            oData.GetDescriptor().toLowerCase().indexOf("juvenile") > -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 1"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 0.1, 0.01);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 0.2, 0.01);
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex + 1);
      
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 0.3, 0.01);
      assertEquals( ( (Number) oVector.GetValue().elementAt(1)).floatValue(),
                   (float) 0.4, 0.01);

      //Change the species
     String[] sNewSpecies = new String[] {
            "Species 2",
            "Species 3",
            "Species 4"};
      oManager.GetTreePopulation().SetSpeciesNames(sNewSpecies);
      oGrowth = oManager.GetGrowthBehaviors();

      //Count the total number of NCIs
      int iCount = 0;
      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1 &&
            oData.GetDescriptor().toLowerCase().indexOf("juvenile") == -1) {
          iCount++;
        }
      }
      assertEquals(3, iCount);

      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1 &&
            oData.GetDescriptor().toLowerCase().indexOf("juvenile") == -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 2"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 0.12, 0.01);

      iIndex++;
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 3") &&
                 -1 != oVector.GetDescriptor().indexOf("NCI Lambda"));

      iIndex++;
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 4") &&
                 -1 != oVector.GetDescriptor().indexOf("NCI Lambda"));

      iIndex++;
      
      
      // ** Repeat with juvenile NCI **
      //Count the total number of NCIs
      iCount = 0;
      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1 &&
            oData.GetDescriptor().toLowerCase().indexOf("juvenile") > -1) {
          iCount++;
        }
      }
      assertEquals(3, iCount);

      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1 &&
            oData.GetDescriptor().toLowerCase().indexOf("juvenile") > -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 2"));
      assertEquals( ( (Number) oVector.GetValue().elementAt(0)).floatValue(),
                   (float) 0.4, 0.01);

      iIndex++;
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 3") &&
                 -1 != oVector.GetDescriptor().indexOf("Juvenile NCI Lambda"));

      iIndex++;
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Species 4") &&
                 -1 != oVector.GetDescriptor().indexOf("Juvenile NCI Lambda"));

      iIndex++;
      if (iIndex < oGrowth.mp_oAllData.size()) {
       oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
       assertTrue(-1 == oVector.GetDescriptor().indexOf("Juvenile NCI Lambda"));
     } 
    }
    catch (ModelException oErr) {
      fail("Growth validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
  }

  /**
   * Makes sure that nci lambda values are managed correctly when species
   * are copied.
   */
  public void testCopySpecies() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      ModelData oData;
      ModelVector oVector;
      int i, iIndex = 0;

      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLFileForSpeciesCopy();
      oManager.InputXMLParameterFile(sFileName);
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1) {
          iIndex = i;
          break;
        }
      }

      //Verify how the data was read in before species copy
      Behavior oBeh = oGrowth.GetBehaviorByKey("nci");
      assertTrue(oBeh.m_bIsEnabled);
      assertEquals(4, oBeh.GetNumberOfCombos());
      SpeciesTypeCombo oCombo;
      for (i = 0; i < oBeh.GetNumberOfCombos(); i++) {
        oCombo = oBeh.GetSpeciesTypeCombo(i);
        assertEquals(3, oCombo.GetType());
        assertEquals(i, oCombo.GetSpecies());
      }

      Float fVal;

      //Max potential growth
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(0);
      assertEquals(1.52, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(1);
      assertEquals(2.5, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(2);
      assertEquals(1.18, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(3);
      assertEquals(1.19, fVal.floatValue(), 0.001);
      
      fVal = (Float)oGrowth.mp_fJuvenileNCIMaxPotentialGrowth.GetValue().elementAt(1);
      assertEquals(1.1, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fJuvenileNCIMaxPotentialGrowth.GetValue().elementAt(6);
      assertEquals(1.2, fVal.floatValue(), 0.001);
    

      //NCI Alpha
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(0);
      assertEquals(2.17, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(1);
      assertEquals(1.86, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(2);
      assertEquals(1.51, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(3);
      assertEquals(1.6, fVal.floatValue(), 0.001);
      
      fVal = (Float)oGrowth.mp_fJuvenileNCIAlpha.GetValue().elementAt(1);
      assertEquals(2.34, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fJuvenileNCIAlpha.GetValue().elementAt(6);
      assertEquals(2.35, fVal.floatValue(), 0.001);
      

      //NCI Beta
      fVal = (Float)oGrowth.mp_fNCIBeta.GetValue().elementAt(0);
      assertEquals(0.69, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIBeta.GetValue().elementAt(1);
      assertEquals(0.46, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIBeta.GetValue().elementAt(2);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIBeta.GetValue().elementAt(3);
      assertEquals(0.28, fVal.floatValue(), 0.001);
      
      fVal = (Float)oGrowth.mp_fJuvenileNCIBeta.GetValue().elementAt(1);
      assertEquals(0.42, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fJuvenileNCIBeta.GetValue().elementAt(6);
      assertEquals(0.57, fVal.floatValue(), 0.001);
      
      //Lambdas
      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1 && 
            oData.GetDescriptor().toLowerCase().indexOf("juvenile") == -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Western Hemlock"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.66, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.37, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.43, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Western Redcedar"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.44, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.49, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(4.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(5.58E-4, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Amabilis Fir"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.21, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.25, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Subalpine Fir"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.21, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Hybrid Spruce"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.27, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.22, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Lodgepole Pine"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.48, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(6.21, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.52, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Trembling Aspen"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.38, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.61, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.38, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.78, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Black Cottonwood"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.33, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.66, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.31, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.87, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Paper Birch"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.74, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.19, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.14, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.88, fVal.floatValue(), 0.001);
      
      
      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1 && 
            oData.GetDescriptor().toLowerCase().indexOf("juvenile") > -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Western Hemlock"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.11, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.12, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Western Redcedar"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.13, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.14, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Amabilis Fir"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.15, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.16, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Subalpine Fir"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.17, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Hybrid Spruce"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.19, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.2, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Lodgepole Pine"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.21, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.22, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Trembling Aspen"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.23, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.24, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Black Cottonwood"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.25, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.26, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Paper Birch"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.27, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.28, fVal.floatValue(), 0.001);
      
      


      //Copy the species
      oManager.GetTreePopulation().DoCopySpecies("Western_Redcedar", "Black_Cottonwood");

      oBeh = oGrowth.GetBehaviorByKey("nci");
      assertTrue(oBeh.m_bIsEnabled);
      assertEquals(5, oBeh.GetNumberOfCombos());
      for (i = 0; i < oBeh.GetNumberOfCombos() - 1; i++) {
        oCombo = oBeh.GetSpeciesTypeCombo(i);
        assertEquals(3, oCombo.GetType());
        assertEquals(i, oCombo.GetSpecies());
      }
      oCombo = oBeh.GetSpeciesTypeCombo(4);
      assertEquals(3, oCombo.GetType());
      assertEquals(7, oCombo.GetSpecies());
      
      oBeh = oGrowth.GetBehaviorByKey("Juvenile NCI");
      assertTrue(oBeh.m_bIsEnabled);
      assertEquals(3, oBeh.GetNumberOfCombos());
      oCombo = oBeh.GetSpeciesTypeCombo(0);
      assertEquals(2, oCombo.GetType());
      assertEquals(1, oCombo.GetSpecies());
      
      oCombo = oBeh.GetSpeciesTypeCombo(1);
      assertEquals(2, oCombo.GetType());
      assertEquals(6, oCombo.GetSpecies());
      
      oCombo = oBeh.GetSpeciesTypeCombo(2);
      assertEquals(2, oCombo.GetType());
      assertEquals(7, oCombo.GetSpecies());

      //Max potential growth
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(0);
      assertEquals(1.52, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(1);
      assertEquals(2.5, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(2);
      assertEquals(1.18, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(3);
      assertEquals(1.19, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(7);
      assertEquals(2.5, fVal.floatValue(), 0.001);
      
      fVal = (Float)oGrowth.mp_fJuvenileNCIMaxPotentialGrowth.GetValue().elementAt(1);
      assertEquals(1.1, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fJuvenileNCIMaxPotentialGrowth.GetValue().elementAt(6);
      assertEquals(1.2, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fJuvenileNCIMaxPotentialGrowth.GetValue().elementAt(7);
      assertEquals(1.1, fVal.floatValue(), 0.001);

      //NCI Alpha
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(0);
      assertEquals(2.17, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(1);
      assertEquals(1.86, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(2);
      assertEquals(1.51, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(3);
      assertEquals(1.6, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(7);
      assertEquals(1.86, fVal.floatValue(), 0.001);
      
      fVal = (Float)oGrowth.mp_fJuvenileNCIAlpha.GetValue().elementAt(1);
      assertEquals(2.34, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fJuvenileNCIAlpha.GetValue().elementAt(6);
      assertEquals(2.35, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fJuvenileNCIAlpha.GetValue().elementAt(7);
      assertEquals(2.34, fVal.floatValue(), 0.001);

      //NCI Beta
      fVal = (Float)oGrowth.mp_fNCIBeta.GetValue().elementAt(0);
      assertEquals(0.69, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIBeta.GetValue().elementAt(1);
      assertEquals(0.46, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIBeta.GetValue().elementAt(2);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIBeta.GetValue().elementAt(3);
      assertEquals(0.28, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIBeta.GetValue().elementAt(7);
      assertEquals(0.46, fVal.floatValue(), 0.001);
      
      fVal = (Float)oGrowth.mp_fJuvenileNCIBeta.GetValue().elementAt(1);
      assertEquals(0.42, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fJuvenileNCIBeta.GetValue().elementAt(6);
      assertEquals(0.57, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fJuvenileNCIBeta.GetValue().elementAt(7);
      assertEquals(0.42, fVal.floatValue(), 0.001);

      //Lambdas
      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1 && 
            oData.GetDescriptor().toLowerCase().indexOf("juvenile") == -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Western Hemlock"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.66, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.37, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.43, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(0.37, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Western Redcedar"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.44, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.49, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(4.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(5.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(0.49, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Amabilis Fir"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.21, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.25, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Subalpine Fir"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.21, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Hybrid Spruce"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.27, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.22, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Lodgepole Pine"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.48, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(6.21, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.52, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(6.21, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Trembling Aspen"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.38, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.61, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.38, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.78, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(0.61, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Black Cottonwood"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.44, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.49, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(4.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(5.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(0.49, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Paper Birch"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.74, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.19, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.14, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.88, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(0.19, fVal.floatValue(), 0.001);
      
      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1 && 
            oData.GetDescriptor().toLowerCase().indexOf("juvenile") > -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Western Hemlock"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.11, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.12, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(0.11, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Western Redcedar"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.13, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.14, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(0.13, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Amabilis Fir"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.15, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.16, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(0.15, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Subalpine Fir"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.17, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(0.17, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Hybrid Spruce"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.19, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.2, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(0.19, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Lodgepole Pine"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.21, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.22, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(0.21, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Trembling Aspen"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.23, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.24, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(0.23, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Black Cottonwood"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.13, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.14, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(0.13, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Paper Birch"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.27, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.28, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(7);
      assertEquals(0.27, fVal.floatValue(), 0.001);

      System.out.println("Test copy species was successful.");
    }
    catch (ModelException oErr) {
      fail("Test copy species failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
  }
  
  /**
   * Makes sure that nci lambda values are managed correctly when species
   * are copied with very short names ("A").
   */
  public void testCopySpecies2() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      ModelData oData;
      ModelVector oVector;
      int i, iIndex = 0;

      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLFileForSpeciesCopy();
      oManager.InputXMLParameterFile(sFileName);
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1) {
          iIndex = i;
          break;
        }
      }

      //Verify how the data was read in before species copy
      Behavior oBeh = oGrowth.GetBehaviorByKey("nci");
      assertTrue(oBeh.m_bIsEnabled);
      assertEquals(4, oBeh.GetNumberOfCombos());
      SpeciesTypeCombo oCombo;
      for (i = 0; i < oBeh.GetNumberOfCombos(); i++) {
        oCombo = oBeh.GetSpeciesTypeCombo(i);
        assertEquals(3, oCombo.GetType());
        assertEquals(i, oCombo.GetSpecies());
      }

      Float fVal;      

      //Create a new species
      String[] sNewSpecies = new String[] {
          "Western Hemlock",
          "Western Redcedar",
          "Amabilis Fir",
          "Subalpine Fir",
          "Hybrid Spruce",
          "Lodgepole Pine",
          "Trembling Aspen",
          "Black Cottonwood",
          "Paper Birch",
          "A"};
      oManager.GetTreePopulation().SetSpeciesNames(sNewSpecies);
      //Copy the species
      oManager.GetTreePopulation().DoCopySpecies("Western_Redcedar", "A");

      oBeh = oGrowth.GetBehaviorByKey("nci");
      assertTrue(oBeh.m_bIsEnabled);
      assertEquals(5, oBeh.GetNumberOfCombos());
      for (i = 0; i < oBeh.GetNumberOfCombos() - 1; i++) {
        oCombo = oBeh.GetSpeciesTypeCombo(i);
        assertEquals(3, oCombo.GetType());
        assertEquals(i, oCombo.GetSpecies());
      }
      oCombo = oBeh.GetSpeciesTypeCombo(4);
      assertEquals(3, oCombo.GetType());
      assertEquals(9, oCombo.GetSpecies());
           
      oBeh = oGrowth.GetBehaviorByKey("Juvenile NCI");
      assertTrue(oBeh.m_bIsEnabled);
      assertEquals(3, oBeh.GetNumberOfCombos());
      oCombo = oBeh.GetSpeciesTypeCombo(0);
      assertEquals(2, oCombo.GetType());
      assertEquals(1, oCombo.GetSpecies());
      
      oCombo = oBeh.GetSpeciesTypeCombo(1);
      assertEquals(2, oCombo.GetType());
      assertEquals(6, oCombo.GetSpecies());
      
      oCombo = oBeh.GetSpeciesTypeCombo(2);
      assertEquals(2, oCombo.GetType());
      assertEquals(9, oCombo.GetSpecies());

      //Max potential growth
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(0);
      assertEquals(1.52, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(1);
      assertEquals(2.5, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(2);
      assertEquals(1.18, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(3);
      assertEquals(1.19, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().elementAt(9);
      assertEquals(2.5, fVal.floatValue(), 0.001);
      
      fVal = (Float)oGrowth.mp_fJuvenileNCIMaxPotentialGrowth.GetValue().elementAt(1);
      assertEquals(1.1, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fJuvenileNCIMaxPotentialGrowth.GetValue().elementAt(6);
      assertEquals(1.2, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fJuvenileNCIMaxPotentialGrowth.GetValue().elementAt(9);
      assertEquals(1.1, fVal.floatValue(), 0.001);
    

      //NCI Alpha
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(0);
      assertEquals(2.17, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(1);
      assertEquals(1.86, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(2);
      assertEquals(1.51, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(3);
      assertEquals(1.6, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fNCIAlpha.GetValue().elementAt(9);
      assertEquals(1.86, fVal.floatValue(), 0.001);
      
      fVal = (Float)oGrowth.mp_fJuvenileNCIAlpha.GetValue().elementAt(1);
      assertEquals(2.34, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fJuvenileNCIAlpha.GetValue().elementAt(6);
      assertEquals(2.35, fVal.floatValue(), 0.001);
      fVal = (Float)oGrowth.mp_fJuvenileNCIAlpha.GetValue().elementAt(9);
      assertEquals(2.34, fVal.floatValue(), 0.001);
            
      //Lambdas
      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1 && 
            oData.GetDescriptor().toLowerCase().indexOf("juvenile") == -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Western Hemlock"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.66, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.37, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.43, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(9);
      assertEquals(0.37, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Western Redcedar"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.44, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.49, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(4.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(5.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(9);
      assertEquals(0.49, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Amabilis Fir"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.21, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.25, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(9);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Subalpine Fir"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.21, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(9);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Hybrid Spruce"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.41, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.27, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.22, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(9);
      assertEquals(6.21E-4, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Lodgepole Pine"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.48, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(6.21, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.52, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(9);
      assertEquals(6.21, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Trembling Aspen"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.38, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.61, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.38, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.78, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(9);
      assertEquals(0.61, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Black Cottonwood"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.33, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.66, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.31, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.87, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(9);
      assertEquals(0.66, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Paper Birch"));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.74, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.19, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(0.14, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(0.88, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(9);
      assertEquals(0.19, fVal.floatValue(), 0.001);
      iIndex++;
      
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("A "));
      fVal = (Float)oVector.GetValue().elementAt(0);
      assertEquals(0.44, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.49, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(2);
      assertEquals(4.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(3);
      assertEquals(5.58E-4, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(9);
      assertEquals(0.49, fVal.floatValue(), 0.001);
      iIndex++;
      
      
      for (i = 0; i < oGrowth.mp_oAllData.size(); i++) {
        oData = (ModelData) oGrowth.mp_oAllData.elementAt(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1 && 
            oData.GetDescriptor().toLowerCase().indexOf("juvenile") > -1) {
          iIndex = i;
          break;
        }
      }
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Western Hemlock"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.11, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.12, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(9);
      assertEquals(0.11, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Western Redcedar"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.13, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.14, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(9);
      assertEquals(0.13, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Amabilis Fir"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.15, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.16, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(9);
      assertEquals(0.15, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Subalpine Fir"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.17, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.18, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(9);
      assertEquals(0.17, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Hybrid Spruce"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.19, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.2, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(9);
      assertEquals(0.19, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Lodgepole Pine"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.21, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.22, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(9);
      assertEquals(0.21, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Trembling Aspen"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.23, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.24, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(9);
      assertEquals(0.23, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Black Cottonwood"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.25, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.26, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(9);
      assertEquals(0.25, fVal.floatValue(), 0.001);
      iIndex++;

      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("Paper Birch"));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.27, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.28, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(9);
      assertEquals(0.27, fVal.floatValue(), 0.001);
      iIndex++;
      
      oVector = (ModelVector) oGrowth.mp_oAllData.elementAt(iIndex);
      assertTrue(-1 != oVector.GetDescriptor().indexOf("A "));
      fVal = (Float)oVector.GetValue().elementAt(1);
      assertEquals(0.13, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(6);
      assertEquals(0.14, fVal.floatValue(), 0.001);
      fVal = (Float)oVector.GetValue().elementAt(9);
      assertEquals(0.13, fVal.floatValue(), 0.001);
      iIndex++;

      System.out.println("Test copy species was successful.");
    }
    catch (ModelException oErr) {
      fail("Test copy species failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
  }

  /**
   * Tests ValidateData().
   */
  public void testValidateData() {
    GUIManager oManager = null;
    String sFileName = null;
    TreePopulation oPop = null;
    try {

      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      new File(sFileName).delete();

      //Valid file 2
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile2();
      oManager.InputXMLParameterFile(sFileName);
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      new File(sFileName).delete();

    }
    catch (ModelException oErr) {
      fail("Growth validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    
    try {
      //NCI max radius of neighbor effects is < 0.
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("nci");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oGrowth.mp_fNCIMaxCrowdingRadius.GetValue().add(0, new Float(-20));
      oGrowth.mp_fNCIMaxCrowdingRadius.GetValue().add(1, new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad NCI max radius values.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
    
    try {
      //NCI max radius of neighbor effects is < 0 - check diam only is caught
      //too...
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("nci diam only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oGrowth.mp_fNCIMaxCrowdingRadius.GetValue().add(0, new Float(-20));
      oGrowth.mp_fNCIMaxCrowdingRadius.GetValue().add(1, new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad NCI max radius values.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
    
    
    try {
      //NCI max growth for a species is < 0.
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("nci");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().add(0, new Float(-20));
      oGrowth.mp_fNCIMaxPotentialGrowth.GetValue().add(1, new Float(20));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad NCI max growth values.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
    
    try {
      //NCI Xb for a species = 0.
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("nci");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oGrowth.mp_fNCISizeEffectVariance.GetValue().add(0, new Float(0));
      oGrowth.mp_fNCISizeEffectVariance.GetValue().add(1, new Float(20));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad NCI Xb for a species = 0.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
    
    try {
      //NCI X0 for a species = 0.
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("nci");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oGrowth.mp_fNCISizeEffectMode.GetValue().add(0, new Float(0));
      oGrowth.mp_fNCISizeEffectMode.GetValue().add(1, new Float(20));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad NCI X0 for a species = 0.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
    
    try {
      //NCI full damage storm effect for a species is not a proportion
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("nci");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oGrowth.mp_fNCIStormEffectFull.GetValue().add(0, new Float(20));
      oGrowth.mp_fNCIStormEffectFull.GetValue().add(1, new Float(0.3));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad NCI full damage storm effect.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
    
    try {
      //NCI med damage storm effect for a species is not a proportion
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("nci");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oGrowth.mp_fNCIStormEffectMed.GetValue().add(0, new Float(20));
      oGrowth.mp_fNCIStormEffectMed.GetValue().add(1, new Float(0.3));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad NCI med damage storm effect.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
    
    try {
      //NCI med damage eta for a species is not a proportion
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("nci");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oGrowth.mp_fNCINeighStormEffMed.GetValue().add(0, new Float(20));
      oGrowth.mp_fNCINeighStormEffMed.GetValue().add(1, new Float(0.3));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad NCI med damage eta.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
    
    try {
      //NCI full damage eta for a species is not a proportion
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("nci");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oGrowth.mp_fNCINeighStormEffFull.GetValue().add(0, new Float(20));
      oGrowth.mp_fNCINeighStormEffFull.GetValue().add(1, new Float(0.3));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad NCI full damage eta.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
    
    try {
      //NCI minimum neighbor DBH is less than 0
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("nci");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oGrowth.mp_fNCIMinNeighborDBH.GetValue().add(0, new Float(-20));
      oGrowth.mp_fNCIMinNeighborDBH.GetValue().add(1, new Float(0.3));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad NCI full damage eta.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
    
     try {
      //Light threshold not between 0 and 100 for linear bi-level growth
      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("linear bi-level");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oGrowth.mp_fBiLevelHighLightThreshold.GetValue().add(new Float(-20));
      oGrowth.mp_fBiLevelHighLightThreshold.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad light threshold values.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //Light threshold not between 0 and 100 for lognormal bi-level growth
      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, oPop));
      oGrowth.mp_fLogBiLevelHighLightThreshold.GetValue().add(new Float(-20));
      oGrowth.mp_fLogBiLevelHighLightThreshold.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad light threshold values.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //Low-light X0 = 0 for lognormal bi-level growth
      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, oPop));
      oGrowth.mp_fLogBiLevelLoLiteX0.GetValue().add(new Float(20));
      oGrowth.mp_fLogBiLevelLoLiteX0.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad X0 value.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //High-light X0 = 0 for lognormal bi-level growth
      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, oPop));
      oGrowth.mp_fLogBiLevelHiLiteX0.GetValue().add(new Float(20));
      oGrowth.mp_fLogBiLevelHiLiteX0.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad X0 value.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //Low-light Xb = 0 for lognormal bi-level growth
      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, oPop));
      oGrowth.mp_fLogBiLevelLoLiteXb.GetValue().add(new Float(20));
      oGrowth.mp_fLogBiLevelLoLiteXb.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad Xb value.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //High-light Xb = 0 for lognormal bi-level growth
      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, oPop));
      oGrowth.mp_fLogBiLevelHiLiteXb.GetValue().add(new Float(20));
      oGrowth.mp_fLogBiLevelHiLiteXb.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad Xb value.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //High-light max growth < 0 for lognormal bi-level growth
      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, oPop));
      oGrowth.mp_fLogBiLevelHiLiteMaxGrwth.GetValue().add(new Float(-20));
      oGrowth.mp_fLogBiLevelHiLiteMaxGrwth.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad max growth value.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //Low-light max growth < 0 for lognormal bi-level growth
      oManager = new GUIManager(null);

      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("lognormal bi-level growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(1, 3, oPop));
      oGrowth.mp_fLogBiLevelLoLiteMaxGrwth.GetValue().add(new Float(-20));
      oGrowth.mp_fLogBiLevelLoLiteMaxGrwth.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad max growth value.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //Valid pr storm bi-level growth situation
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("pr storm bi-level");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oPop));

      //Enable storms and storm light
      oBeh = oManager.GetDisturbanceBehaviors().GetBehaviorByKey("Storms");
      oBeh.m_bIsEnabled = true;
      oBeh = oManager.GetLightBehaviors().GetBehaviorByKey("storm light");
      oBeh.m_bIsEnabled = true;

      oGrowth.mp_fPRStormLevelHighLightThreshold.GetValue().add(new Float(20));
      oGrowth.mp_fPRStormLevelHighLightThreshold.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
    }
    catch (ModelException oErr) {
      fail("Growth validation failed with message: " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }


    try {
      //Light threshold not between 0 and 100 for pr storm bi-level growth
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("pr storm bi-level");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oPop));

      //Enable storms and storm light
      oBeh = oManager.GetDisturbanceBehaviors().GetBehaviorByKey("Storms");
      oBeh.m_bIsEnabled = true;
      oBeh = oManager.GetLightBehaviors().GetBehaviorByKey("storm light");
      oBeh.m_bIsEnabled = true;

      oGrowth.mp_fPRStormLevelHighLightThreshold.GetValue().add(new Float(-20));
      oGrowth.mp_fPRStormLevelHighLightThreshold.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to catch bad light threshold values.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //No storm behavior with pr storm bi-level growth
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("pr storm bi-level");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oPop));

      //Enable storm light
      oBeh = oManager.GetLightBehaviors().GetBehaviorByKey("storm light");
      oBeh.m_bIsEnabled = true;

      oGrowth.mp_fPRStormLevelHighLightThreshold.GetValue().add(new Float(20));
      oGrowth.mp_fPRStormLevelHighLightThreshold.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to notice lack of storms with pr storm bi-level growth.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //No storm light behavior with pr storm bi-level growth
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("pr storm bi-level");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oPop));

      //Enable storms
      oBeh = oManager.GetDisturbanceBehaviors().GetBehaviorByKey("Storms");
      oBeh.m_bIsEnabled = true;

      oGrowth.mp_fBiLevelHighLightThreshold.GetValue().add(new Float(20));
      oGrowth.mp_fBiLevelHighLightThreshold.GetValue().add(new Float(0));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to notice lack of storm light with pr storm bi-level growth.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
    
    try {
      //Negative beta for Michaelis Menten negative growth 
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("Michaelis Menton with negative growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oPop));

      oGrowth.mp_fMMNegGrowthBeta.GetValue().add(new Float(0));
      oGrowth.mp_fMMNegGrowthBeta.GetValue().add(new Float(10));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to notice beta equal to zero for Michaelis Menten negative growth.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
    
    try {
      //Bad autocorrelation prob for Michaelis Menten negative growth 
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("Michaelis Menton with negative growth height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oPop));

      oGrowth.mp_fMMNegGrowthAutoCorrProb.GetValue().add(new Float(10));
      oGrowth.mp_fMMNegGrowthAutoCorrProb.GetValue().add(new Float(10));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to notice bad autocorrelation probability for Michaelis Menten negative growth.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
    
    try {
      //Negative beta for Michaelis Menten photoinhibition growth 
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteXMLValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPop = oManager.GetTreePopulation();
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      Behavior oBeh = oGrowth.GetBehaviorByKey("Michaelis Menton with photoinhibition height only");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oPop));

      oGrowth.mp_fMMPhotGrowthBeta.GetValue().add(new Float(0));
      oGrowth.mp_fMMPhotGrowthBeta.GetValue().add(new Float(10));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to notice beta equal to zero for Michaelis Menten negative growth.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }
    
    try {
      //Weibull climate growth - Max radius of neighbor effects is < 0 
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteXMLValidFile1());
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      Behavior oBeh = oManager.GetGrowthBehaviors().GetBehaviorByKey("Weibull climate");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oManager.GetTreePopulation()));

      oGrowth.mp_fWeibClimMaxCrowdingRadius.GetValue().add(new Float(-5));
      oGrowth.mp_fWeibClimMaxCrowdingRadius.GetValue().add(new Float(10));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to notice Max radius of neighbor effects is < 0 for Weibull climate growth.");
    }
    catch (ModelException oErr) {;}
    catch (java.io.IOException oE) { fail("Caught IOException.  Message: " + oE.getMessage());}
    finally {new File(sFileName).delete();}
    
    try {
      //Weibull climate growth - Max growth for any species is < 0
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteXMLValidFile1());
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      Behavior oBeh = oManager.GetGrowthBehaviors().GetBehaviorByKey("Weibull climate");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oManager.GetTreePopulation()));
      oGrowth.mp_fWeibClimMaxRG.GetValue().clear();
      oGrowth.mp_fWeibClimMaxRG.GetValue().add(new Float(-5));
      oGrowth.mp_fWeibClimMaxRG.GetValue().add(new Float(10));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to notice Max growth for any species is < 0 for Weibull climate growth.");
    }
    catch (ModelException oErr) {;}
    catch (java.io.IOException oE) { fail("Caught IOException.  Message: " + oE.getMessage());}
    finally {new File(sFileName).delete();}
    
    try {
      //Weibull climate growth - adult temp A for a species = 0
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteXMLValidFile1());
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      Behavior oBeh = oManager.GetGrowthBehaviors().GetBehaviorByKey("Weibull climate");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oManager.GetTreePopulation()));
      oGrowth.mp_fWeibClimAdultTempA.GetValue().clear();
      oGrowth.mp_fWeibClimAdultTempA.GetValue().add(new Float(0));
      oGrowth.mp_fWeibClimAdultTempA.GetValue().add(new Float(10));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to notice adult temp A for a species = 0 for Weibull climate growth.");
    }
    catch (ModelException oErr) {;}
    catch (java.io.IOException oE) { fail("Caught IOException.  Message: " + oE.getMessage());}
    finally {new File(sFileName).delete();}
    
    try {
      //Weibull climate growth - adult precip A for a species = 0
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteXMLValidFile1());
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      Behavior oBeh = oManager.GetGrowthBehaviors().GetBehaviorByKey("Weibull climate");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oManager.GetTreePopulation()));
      oGrowth.mp_fWeibClimAdultPrecipA.GetValue().clear();
      oGrowth.mp_fWeibClimAdultPrecipA.GetValue().add(new Float(0));
      oGrowth.mp_fWeibClimAdultPrecipA.GetValue().add(new Float(10));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to notice adult precip A for a species = 0 for Weibull climate growth.");
    }
    catch (ModelException oErr) {;}
    catch (java.io.IOException oE) { fail("Caught IOException.  Message: " + oE.getMessage());}
    finally {new File(sFileName).delete();}

    try {
      //Weibull climate growth - juvenile temp A for a species = 0
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteXMLValidFile1());
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      Behavior oBeh = oManager.GetGrowthBehaviors().GetBehaviorByKey("Weibull climate");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oManager.GetTreePopulation()));
      oGrowth.mp_fWeibClimJuvenileTempA.GetValue().clear();
      oGrowth.mp_fWeibClimJuvenileTempA.GetValue().add(new Float(0));
      oGrowth.mp_fWeibClimJuvenileTempA.GetValue().add(new Float(10));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to notice Juvenile temp A for a species = 0 for Weibull climate growth.");
    }
    catch (ModelException oErr) {;}
    catch (java.io.IOException oE) { fail("Caught IOException.  Message: " + oE.getMessage());}
    finally {new File(sFileName).delete();}

    try {
      //Weibull climate growth - Juvenile precip A for a species = 0
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteXMLValidFile1());
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      Behavior oBeh = oManager.GetGrowthBehaviors().GetBehaviorByKey("Weibull climate");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oManager.GetTreePopulation()));
      oGrowth.mp_fWeibClimJuvenilePrecipA.GetValue().clear();
      oGrowth.mp_fWeibClimJuvenilePrecipA.GetValue().add(new Float(0));
      oGrowth.mp_fWeibClimJuvenilePrecipA.GetValue().add(new Float(10));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to notice Juvenile precip A for a species = 0 for Weibull climate growth.");
    }
    catch (ModelException oErr) {;}
    catch (java.io.IOException oE) { fail("Caught IOException.  Message: " + oE.getMessage());}
    finally {new File(sFileName).delete();}
    
    try {
      //Weibull climate growth - X0 (size effect mode) for any species = 0
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteXMLValidFile1());
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      Behavior oBeh = oManager.GetGrowthBehaviors().GetBehaviorByKey("Weibull climate");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oManager.GetTreePopulation()));
      oGrowth.mp_fWeibClimSizeX0.GetValue().clear();
      oGrowth.mp_fWeibClimSizeX0.GetValue().add(new Float(0));
      oGrowth.mp_fWeibClimSizeX0.GetValue().add(new Float(10));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to notice X0 (size effect mode) for any species = 0 for Weibull climate growth.");
    }
    catch (ModelException oErr) {;}
    catch (java.io.IOException oE) { fail("Caught IOException.  Message: " + oE.getMessage());}
    finally {new File(sFileName).delete();}
    
    try {
      //Weibull climate growth - Xb (size effect mode) for any species = 0
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteXMLValidFile1());
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      Behavior oBeh = oManager.GetGrowthBehaviors().GetBehaviorByKey("Weibull climate");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oManager.GetTreePopulation()));
      oGrowth.mp_fWeibClimSizeXb.GetValue().clear();
      oGrowth.mp_fWeibClimSizeXb.GetValue().add(new Float(0));
      oGrowth.mp_fWeibClimSizeXb.GetValue().add(new Float(10));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to notice Xb (size effect mode) for any species = 0 for Weibull climate growth.");
    }
    catch (ModelException oErr) {;}
    catch (java.io.IOException oE) { fail("Caught IOException.  Message: " + oE.getMessage());}
    finally {new File(sFileName).delete();}
    
    try {
      //Weibull climate quadrat growth - Max radius of neighbor effects is < 0 
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteXMLValidFile1());
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      Behavior oBeh = oManager.GetGrowthBehaviors().GetBehaviorByKey("Weibull climate quadrat growth");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oManager.GetTreePopulation()));
      oGrowth.m_fWeibClimQuadMaxCrowdingRadius.SetValue(new Float (-5)); 
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to notice Max radius of neighbor effects is < 0 for Weibull climate quadrat growth.");
    }
    catch (ModelException oErr) {;}
    catch (java.io.IOException oE) { fail("Caught IOException.  Message: " + oE.getMessage());}
    finally {new File(sFileName).delete();}
    
    try {
      //Weibull climate quadrat growth - Max growth for any species is < 0
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteXMLValidFile1());
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      Behavior oBeh = oManager.GetGrowthBehaviors().GetBehaviorByKey("Weibull climate quadrat growth");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oManager.GetTreePopulation()));
      oGrowth.mp_fWeibClimQuadMaxRG.GetValue().clear();
      oGrowth.mp_fWeibClimQuadMaxRG.GetValue().add(new Float(-5));
      oGrowth.mp_fWeibClimQuadMaxRG.GetValue().add(new Float(10));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to notice Max growth for any species is < 0 for Weibull climate quadrat growth.");
    }
    catch (ModelException oErr) {;}
    catch (java.io.IOException oE) { fail("Caught IOException.  Message: " + oE.getMessage());}
    finally {new File(sFileName).delete();}
    
    try {
      //Weibull climate quadrat growth - temp A for a species = 0
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteXMLValidFile1());
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      Behavior oBeh = oManager.GetGrowthBehaviors().GetBehaviorByKey("Weibull climate quadrat growth");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oManager.GetTreePopulation()));
      oGrowth.mp_fWeibClimQuadTempA.GetValue().clear();
      oGrowth.mp_fWeibClimQuadTempA.GetValue().add(new Float(0));
      oGrowth.mp_fWeibClimQuadTempA.GetValue().add(new Float(10));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to notice temp A for a species = 0 for Weibull climate quadrat growth.");
    }
    catch (ModelException oErr) {;}
    catch (java.io.IOException oE) { fail("Caught IOException.  Message: " + oE.getMessage());}
    finally {new File(sFileName).delete();}
    
    try {
      //Weibull climate quadrat growth - precip A for a species = 0
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteXMLValidFile1());
      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();
      Behavior oBeh = oManager.GetGrowthBehaviors().GetBehaviorByKey("Weibull climate quadrat growth");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 1, oManager.GetTreePopulation()));
      oGrowth.mp_fWeibClimQuadPrecipA.GetValue().clear();
      oGrowth.mp_fWeibClimQuadPrecipA.GetValue().add(new Float(0));
      oGrowth.mp_fWeibClimQuadPrecipA.GetValue().add(new Float(10));
      oManager.GetGrowthBehaviors().ValidateData(oManager.GetTreePopulation());
      fail("Growth validation failed to notice precip A for a species = 0 for Weibull climate quadrat growth.");
    }
    catch (ModelException oErr) {;}
    catch (java.io.IOException oE) { fail("Caught IOException.  Message: " + oE.getMessage());}
    finally {new File(sFileName).delete();}
    
    System.out.println("Growth ValidateData testing succeeded.");
  }

  /**
   * Writes a valid NCI growth file, Diam with auto height.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteXMLValidFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamAuto(oOut);
    WriteValidFileEnd(oOut);

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a valid NCI growth file, Diam only.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteXMLValidFile2() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    WriteBehaviorListDiamOnly(oOut);
    WriteValidFileEnd(oOut);

    oOut.close();
    return sFileName;
  }

  /**
   * Writes the common first part of parameter files.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteValidateFilePart1(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>1</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">39.48</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</allometry>");
  }

  /**
   * Writes a behavior list for diameter NCI growth with auto height.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteBehaviorListDiamAuto(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>ncigrowth</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>NCI Juvenile Growth</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>WeibullClimateGrowth</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
  }

  /**
   * Writes a behavior list for diameter only NCI growth.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteBehaviorListDiamOnly(FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>ncigrowth diam only</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
  }

  /**
   * Writes the end for a valid parameter file.
   * @throws IOException if there is a problem writing the file.
   * @param oOut File stream to write to.
   */
  private void WriteValidFileEnd(FileWriter oOut) throws java.io.IOException {
    oOut.write("<growth>");
    oOut.write("<gr_nciMaxCrowdingRadius>");
    oOut.write("<gr_nmcrVal species=\"Species_1\">10</gr_nmcrVal>");
    oOut.write("<gr_nmcrVal species=\"Species_2\">10</gr_nmcrVal>");
    oOut.write("</gr_nciMaxCrowdingRadius>");
    oOut.write("<gr_nciAlpha>");
    oOut.write("<gr_naVal species=\"Species_1\">2.17031683</gr_naVal>");
    oOut.write("<gr_naVal species=\"Species_2\">2.81</gr_naVal>");
    oOut.write("</gr_nciAlpha>");
    oOut.write("<gr_nciBeta>");
    oOut.write("<gr_nbVal species=\"Species_1\">0.699942</gr_nbVal>");
    oOut.write("<gr_nbVal species=\"Species_2\">0.5</gr_nbVal>");
    oOut.write("</gr_nciBeta>");
    oOut.write("<gr_nciMaxPotentialGrowth>");
    oOut.write("<gr_nmpgVal species=\"Species_1\">10.5258749</gr_nmpgVal>");
    oOut.write("<gr_nmpgVal species=\"Species_2\">15.31</gr_nmpgVal>");
    oOut.write("</gr_nciMaxPotentialGrowth>");
    oOut.write("<gr_nciSizeSensNCI>");
    oOut.write("<gr_nssnVal species=\"Species_1\">0.43</gr_nssnVal>");
    oOut.write("<gr_nssnVal species=\"Species_2\">0.36</gr_nssnVal>");
    oOut.write("</gr_nciSizeSensNCI>");
    oOut.write("<gr_nciSizeEffectMode>");
    oOut.write("<gr_nsemVal species=\"Species_1\">34.24638994</gr_nsemVal>");
    oOut.write("<gr_nsemVal species=\"Species_2\">122.23</gr_nsemVal>");
    oOut.write("</gr_nciSizeEffectMode>");
    oOut.write("<gr_nciSizeEffectVariance>");
    oOut.write("<gr_nsevVal species=\"Species_1\">5</gr_nsevVal>");
    oOut.write("<gr_nsevVal species=\"Species_2\">2.36</gr_nsevVal>");
    oOut.write("</gr_nciSizeEffectVariance>");
    oOut.write("<gr_nciCrowdingSlope>");
    oOut.write("<gr_ncslVal species=\"Species_1\">1.98</gr_ncslVal>");
    oOut.write("<gr_ncslVal species=\"Species_2\">4.57</gr_ncslVal>");
    oOut.write("</gr_nciCrowdingSlope>");
    oOut.write("<gr_nciCrowdingSteepness>");
    oOut.write("<gr_ncstVal species=\"Species_1\">1.63</gr_ncstVal>");
    oOut.write("<gr_ncstVal species=\"Species_2\">1.26</gr_ncstVal>");
    oOut.write("</gr_nciCrowdingSteepness>");
    oOut.write("<gr_nciStormEffMedDmg>");
    oOut.write("<gr_nsemdVal species=\"Species_1\">0.89</gr_nsemdVal>");
    oOut.write("<gr_nsemdVal species=\"Species_2\">0.9</gr_nsemdVal>");
    oOut.write("</gr_nciStormEffMedDmg>");
    oOut.write("<gr_nciStormEffFullDmg>");
    oOut.write("<gr_nsefdVal species=\"Species_1\">0.42</gr_nsefdVal>");
    oOut.write("<gr_nsefdVal species=\"Species_2\">0.34</gr_nsefdVal>");
    oOut.write("</gr_nciStormEffFullDmg>");
    oOut.write("<gr_nciNeighStormEffMedDmg>");
    oOut.write("<gr_nnsemdVal species=\"Species_1\">0.89</gr_nnsemdVal>");
    oOut.write("<gr_nnsemdVal species=\"Species_2\">0.9</gr_nnsemdVal>");
    oOut.write("</gr_nciNeighStormEffMedDmg>");
    oOut.write("<gr_nciNeighStormEffFullDmg>");
    oOut.write("<gr_nnsefdVal species=\"Species_1\">0.42</gr_nnsefdVal>");
    oOut.write("<gr_nnsefdVal species=\"Species_2\">0.34</gr_nnsefdVal>");
    oOut.write("</gr_nciNeighStormEffFullDmg>");
    oOut.write("<gr_nciSpecies_1NeighborLambda>");
    oOut.write("<gr_nlVal species=\"Species_1\">0.6640108</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Species_2\">0.71</gr_nlVal>");
    oOut.write("</gr_nciSpecies_1NeighborLambda>");
    oOut.write("<gr_nciSpecies_2NeighborLambda>");
    oOut.write("<gr_nlVal species=\"Species_1\">0.00442797</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Species_2\">0.12</gr_nlVal>");
    oOut.write("</gr_nciSpecies_2NeighborLambda>");
    oOut.write("<gr_nciShadingCoefficient>");
    oOut.write("<gr_nscVal species=\"Species_1\">0.42</gr_nscVal>");
    oOut.write("<gr_nscVal species=\"Species_2\">0.34</gr_nscVal>");
    oOut.write("</gr_nciShadingCoefficient>");
    oOut.write("<gr_nciShadingExponent>");
    oOut.write("<gr_nseVal species=\"Species_1\">0.42</gr_nseVal>");
    oOut.write("<gr_nseVal species=\"Species_2\">0.34</gr_nseVal>");
    oOut.write("</gr_nciShadingExponent>");
    oOut.write("<gr_nciMinNeighborDBH>");
    oOut.write("<gr_nmndVal species=\"Species_1\">10</gr_nmndVal>");
    oOut.write("<gr_nmndVal species=\"Species_2\">10</gr_nmndVal>");
    oOut.write("</gr_nciMinNeighborDBH>");
    oOut.write("<gr_nciDbhDivisor>1</gr_nciDbhDivisor>");
    
    oOut.write("<gr_juvNCIMaxCrowdingRadius>");
    oOut.write("<gr_jnmcrVal species=\"Species_1\">12</gr_jnmcrVal>");
    oOut.write("<gr_jnmcrVal species=\"Species_2\">12</gr_jnmcrVal>");
    oOut.write("</gr_juvNCIMaxCrowdingRadius>");
    oOut.write("<gr_juvNCIAlpha>");
    oOut.write("<gr_jnaVal species=\"Species_1\">2.17031683</gr_jnaVal>");
    oOut.write("<gr_jnaVal species=\"Species_2\">2.81</gr_jnaVal>");
    oOut.write("</gr_juvNCIAlpha>");
    oOut.write("<gr_juvNCIBeta>");
    oOut.write("<gr_jnbVal species=\"Species_1\">0.699942</gr_jnbVal>");
    oOut.write("<gr_jnbVal species=\"Species_2\">0.5</gr_jnbVal>");
    oOut.write("</gr_juvNCIBeta>");
    oOut.write("<gr_juvNCIMaxPotentialGrowth>");
    oOut.write("<gr_jnmpgVal species=\"Species_1\">1.05258749</gr_jnmpgVal>");
    oOut.write("<gr_jnmpgVal species=\"Species_2\">1.531</gr_jnmpgVal>");
    oOut.write("</gr_juvNCIMaxPotentialGrowth>");
    oOut.write("<gr_juvNCISizeEffectA>");
    oOut.write("<gr_jnseaVal species=\"Species_1\">0.342463899</gr_jnseaVal>");
    oOut.write("<gr_jnseaVal species=\"Species_2\">0.85</gr_jnseaVal>");
    oOut.write("</gr_juvNCISizeEffectA>");
    oOut.write("<gr_juvNCISizeEffectB>");
    oOut.write("<gr_jnsebVal species=\"Species_1\">0.05</gr_jnsebVal>");
    oOut.write("<gr_jnsebVal species=\"Species_2\">0.0236</gr_jnsebVal>");
    oOut.write("</gr_juvNCISizeEffectB>");
    oOut.write("<gr_juvNCICrowdingSlope>");
    oOut.write("<gr_jncslVal species=\"Species_1\">0.698</gr_jncslVal>");
    oOut.write("<gr_jncslVal species=\"Species_2\">0.457</gr_jncslVal>");
    oOut.write("</gr_juvNCICrowdingSlope>");
    oOut.write("<gr_juvNCICrowdingSteepness>");
    oOut.write("<gr_jncstVal species=\"Species_1\">-0.00163</gr_jncstVal>");
    oOut.write("<gr_jncstVal species=\"Species_2\">-0.00126</gr_jncstVal>");
    oOut.write("</gr_juvNCICrowdingSteepness>");
    oOut.write("<gr_juvNCISpecies_1NeighborLambda>");
    oOut.write("<gr_jnlVal species=\"Species_1\">0.1</gr_jnlVal>");
    oOut.write("<gr_jnlVal species=\"Species_2\">0.2</gr_jnlVal>");
    oOut.write("</gr_juvNCISpecies_1NeighborLambda>");
    oOut.write("<gr_juvNCISpecies_2NeighborLambda>");
    oOut.write("<gr_jnlVal species=\"Species_1\">0.3</gr_jnlVal>");
    oOut.write("<gr_jnlVal species=\"Species_2\">0.4</gr_jnlVal>");
    oOut.write("</gr_juvNCISpecies_2NeighborLambda>");
    oOut.write("<gr_juvNCIMinNeighborDiam10>");
    oOut.write("<gr_jnmndVal species=\"Species_1\">5</gr_jnmndVal>");
    oOut.write("<gr_jnmndVal species=\"Species_2\">5</gr_jnmndVal>");
    oOut.write("</gr_juvNCIMinNeighborDiam10>");
    oOut.write("<gr_juvNCIDiam10Divisor>100</gr_juvNCIDiam10Divisor>");
    oOut.write("<gr_juvNCIIncludeSnagsInNCI>0</gr_juvNCIIncludeSnagsInNCI>");
    oOut.write("<gr_weibClimMaxNeighRad>");
    oOut.write("<gr_wcmnrVal species=\"Species_1\">10</gr_wcmnrVal>");
    oOut.write("<gr_wcmnrVal species=\"Species_2\">11</gr_wcmnrVal>");
    oOut.write("</gr_weibClimMaxNeighRad>");
    oOut.write("<gr_weibClimMaxGrowth>");
    oOut.write("<gr_wcmgVal species=\"Species_1\">0.2574</gr_wcmgVal>");
    oOut.write("<gr_wcmgVal species=\"Species_2\">10</gr_wcmgVal>");
    oOut.write("</gr_weibClimMaxGrowth>");
    oOut.write("<gr_weibClimSizeEffX0>");
    oOut.write("<gr_wcsex0Val species=\"Species_1\">297.615</gr_wcsex0Val>");
    oOut.write("<gr_wcsex0Val species=\"Species_2\">18.07</gr_wcsex0Val>");
    oOut.write("</gr_weibClimSizeEffX0>");
    oOut.write("<gr_weibClimSizeEffXb>");
    oOut.write("<gr_wcsexbVal species=\"Species_1\">4.065</gr_wcsexbVal>");
    oOut.write("<gr_wcsexbVal species=\"Species_2\">0.902</gr_wcsexbVal>");
    oOut.write("</gr_weibClimSizeEffXb>");
    oOut.write("<gr_weibClimAdultCompEffC>");
    oOut.write("<gr_wcacecVal species=\"Species_1\">8.261</gr_wcacecVal>");
    oOut.write("<gr_wcacecVal species=\"Species_2\">0</gr_wcacecVal>");
    oOut.write("</gr_weibClimAdultCompEffC>");
    oOut.write("<gr_weibClimAdultCompEffD>");
    oOut.write("<gr_wcacedVal species=\"Species_1\">1.001</gr_wcacedVal>");
    oOut.write("<gr_wcacedVal species=\"Species_2\">2.914</gr_wcacedVal>");
    oOut.write("</gr_weibClimAdultCompEffD>");
    oOut.write("<gr_weibClimAdultCompEffGamma>");
    oOut.write("<gr_wcacegVal species=\"Species_1\">-1.208</gr_wcacegVal>");
    oOut.write("<gr_wcacegVal species=\"Species_2\">1.976</gr_wcacegVal>");
    oOut.write("</gr_weibClimAdultCompEffGamma>");
    oOut.write("<gr_weibClimJuvCompEffC>");
    oOut.write("<gr_wcjcecVal species=\"Species_1\">15.16225</gr_wcjcecVal>");
    oOut.write("<gr_wcjcecVal species=\"Species_2\">10.67038</gr_wcjcecVal>");
    oOut.write("</gr_weibClimJuvCompEffC>");
    oOut.write("<gr_weibClimJuvCompEffD>");
    oOut.write("<gr_wcjcedVal species=\"Species_1\">0.9</gr_wcjcedVal>");
    oOut.write("<gr_wcjcedVal species=\"Species_2\">1.1</gr_wcjcedVal>");
    oOut.write("</gr_weibClimJuvCompEffD>");
    oOut.write("<gr_weibClimJuvCompEffGamma>");
    oOut.write("<gr_wcjcegVal species=\"Species_1\">-0.06</gr_wcjcegVal>");
    oOut.write("<gr_wcjcegVal species=\"Species_2\">-0.4</gr_wcjcegVal>");
    oOut.write("</gr_weibClimJuvCompEffGamma>");
    oOut.write("<gr_weibClimAdultTempEffA>");
    oOut.write("<gr_wcateaVal species=\"Species_1\">8.8</gr_wcateaVal>");
    oOut.write("<gr_wcateaVal species=\"Species_2\">8.9</gr_wcateaVal>");
    oOut.write("</gr_weibClimAdultTempEffA>");
    oOut.write("<gr_weibClimAdultTempEffB>");
    oOut.write("<gr_wcatebVal species=\"Species_1\">1.2</gr_wcatebVal>");
    oOut.write("<gr_wcatebVal species=\"Species_2\">1.47</gr_wcatebVal>");
    oOut.write("</gr_weibClimAdultTempEffB>");
    oOut.write("<gr_weibClimAdultTempEffC>");
    oOut.write("<gr_wcatecVal species=\"Species_1\">13.2</gr_wcatecVal>");
    oOut.write("<gr_wcatecVal species=\"Species_2\">12.4</gr_wcatecVal>");
    oOut.write("</gr_weibClimAdultTempEffC>");
    oOut.write("<gr_weibClimAdultPrecipEffA>");
    oOut.write("<gr_wcapeaVal species=\"Species_1\">420.2</gr_wcapeaVal>");
    oOut.write("<gr_wcapeaVal species=\"Species_2\">3.7</gr_wcapeaVal>");
    oOut.write("</gr_weibClimAdultPrecipEffA>");
    oOut.write("<gr_weibClimAdultPrecipEffB>");
    oOut.write("<gr_wcapebVal species=\"Species_1\">0.4</gr_wcapebVal>");
    oOut.write("<gr_wcapebVal species=\"Species_2\">0.07</gr_wcapebVal>");
    oOut.write("</gr_weibClimAdultPrecipEffB>");
    oOut.write("<gr_weibClimAdultPrecipEffC>");
    oOut.write("<gr_wcapecVal species=\"Species_1\">0.03</gr_wcapecVal>");
    oOut.write("<gr_wcapecVal species=\"Species_2\">1048.7</gr_wcapecVal>");
    oOut.write("</gr_weibClimAdultPrecipEffC>");
    oOut.write("<gr_weibClimJuvPrecipEffA>");
    oOut.write("<gr_wcjpeaVal species=\"Species_1\">1296.2</gr_wcjpeaVal>");
    oOut.write("<gr_wcjpeaVal species=\"Species_2\">3306.1</gr_wcjpeaVal>");
    oOut.write("</gr_weibClimJuvPrecipEffA>");
    oOut.write("<gr_weibClimJuvPrecipEffB>");
    oOut.write("<gr_wcjpebVal species=\"Species_1\">0.8</gr_wcjpebVal>");
    oOut.write("<gr_wcjpebVal species=\"Species_2\">6.6</gr_wcjpebVal>");
    oOut.write("</gr_weibClimJuvPrecipEffB>");
    oOut.write("<gr_weibClimJuvPrecipEffC>");
    oOut.write("<gr_wcjpecVal species=\"Species_1\">1032.2</gr_wcjpecVal>");
    oOut.write("<gr_wcjpecVal species=\"Species_2\">3835.9</gr_wcjpecVal>");
    oOut.write("</gr_weibClimJuvPrecipEffC>");
    oOut.write("<gr_weibClimJuvTempEffA>");
    oOut.write("<gr_wcjteaVal species=\"Species_1\">6.8</gr_wcjteaVal>");
    oOut.write("<gr_wcjteaVal species=\"Species_2\">139.5</gr_wcjteaVal>");
    oOut.write("</gr_weibClimJuvTempEffA>");
    oOut.write("<gr_weibClimJuvTempEffB>");
    oOut.write("<gr_wcjtebVal species=\"Species_1\">0.3</gr_wcjtebVal>");
    oOut.write("<gr_wcjtebVal species=\"Species_2\">0</gr_wcjtebVal>");
    oOut.write("</gr_weibClimJuvTempEffB>");
    oOut.write("<gr_weibClimJuvTempEffC>");
    oOut.write("<gr_wcjtecVal species=\"Species_1\">14.2</gr_wcjtecVal>");
    oOut.write("<gr_wcjtecVal species=\"Species_2\">28.6</gr_wcjtecVal>");
    oOut.write("</gr_weibClimJuvTempEffC>");
    oOut.write("<gr_weibClimMinNeighDBH>");
    oOut.write("<gr_wcmndVal species=\"Species_1\">5</gr_wcmndVal>");
    oOut.write("<gr_wcmndVal species=\"Species_2\">6</gr_wcmndVal>");
    oOut.write("</gr_weibClimMinNeighDBH>"); 
    oOut.write("</growth>"); 
    oOut.write("</paramFile>");
  }

  /**
   * Writes a file with no growth settings.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteNoGrowthXMLFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>densityselfthinning</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<mortality>");
    oOut.write("<mo_selfThinRadius>");
    oOut.write("<mo_strVal species=\"Species_1\">9</mo_strVal>");
    oOut.write("</mo_selfThinRadius>");
    oOut.write("<mo_minDensityForMort>");
    oOut.write("<mo_mdfmVal species=\"Species_1\">0</mo_mdfmVal>");
    oOut.write("</mo_minDensityForMort>");
    oOut.write("<mo_selfThinAsymptote>");
    oOut.write("<mo_staVal species=\"Species_1\">0.1019</mo_staVal>");
    oOut.write("</mo_selfThinAsymptote>");
    oOut.write("<mo_selfThinDiamEffect>");
    oOut.write("<mo_stdieVal species=\"Species_1\">0.5391</mo_stdieVal>");
    oOut.write("</mo_selfThinDiamEffect>");
    oOut.write("<mo_selfThinDensityEffect>");
    oOut.write("<mo_stdeeVal species=\"Species_1\">0.00000877</mo_stdeeVal>");
    oOut.write("</mo_selfThinDensityEffect>");
    oOut.write("</mortality>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  protected String WriteXMLFileForSpeciesCopy() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>1</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Western_Hemlock\" />");
    oOut.write("<tr_species speciesName=\"Western_Redcedar\" />");
    oOut.write("<tr_species speciesName=\"Amabilis_Fir\" />");
    oOut.write("<tr_species speciesName=\"Subalpine_Fir\" />");
    oOut.write("<tr_species speciesName=\"Hybrid_Spruce\" />");
    oOut.write("<tr_species speciesName=\"Lodgepole_Pine\" />");
    oOut.write("<tr_species speciesName=\"Trembling_Aspen\" />");
    oOut.write("<tr_species speciesName=\"Black_Cottonwood\" />");
    oOut.write("<tr_species speciesName=\"Paper_Birch\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("</trees>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>ncigrowth</behaviorName>");
    oOut.write("<version>2.1</version>");
    oOut.write("<applyTo species=\"Western_Hemlock\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Western_Redcedar\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Amabilis_Fir\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Subalpine_Fir\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>NCI Juvenile Growth</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Western_Redcedar\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Trembling_Aspen\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<growth>");
    oOut.write("<gr_nciMaxPotentialGrowth>");
    oOut.write("<gr_nmpgVal species=\"Western_Hemlock\">1.52</gr_nmpgVal>");
    oOut.write("<gr_nmpgVal species=\"Western_Redcedar\">2.5</gr_nmpgVal>");
    oOut.write("<gr_nmpgVal species=\"Amabilis_Fir\">1.18</gr_nmpgVal>");
    oOut.write("<gr_nmpgVal species=\"Subalpine_Fir\">1.19</gr_nmpgVal>");
    oOut.write("</gr_nciMaxPotentialGrowth>");
    oOut.write("<gr_nciMaxCrowdingRadius>");
    oOut.write("<gr_nmcrVal species=\"Western_Hemlock\">10.0</gr_nmcrVal>");
    oOut.write("<gr_nmcrVal species=\"Western_Redcedar\">10.0</gr_nmcrVal>");
    oOut.write("<gr_nmcrVal species=\"Amabilis_Fir\">10.0</gr_nmcrVal>");
    oOut.write("<gr_nmcrVal species=\"Subalpine_Fir\">10.0</gr_nmcrVal>");
    oOut.write("</gr_nciMaxCrowdingRadius>");
    oOut.write("<gr_nciAlpha>");
    oOut.write("<gr_naVal species=\"Western_Hemlock\">2.17</gr_naVal>");
    oOut.write("<gr_naVal species=\"Western_Redcedar\">1.86</gr_naVal>");
    oOut.write("<gr_naVal species=\"Amabilis_Fir\">1.51</gr_naVal>");
    oOut.write("<gr_naVal species=\"Subalpine_Fir\">1.6</gr_naVal>");
    oOut.write("</gr_nciAlpha>");
    oOut.write("<gr_nciBeta>");
    oOut.write("<gr_nbVal species=\"Western_Hemlock\">0.69</gr_nbVal>");
    oOut.write("<gr_nbVal species=\"Western_Redcedar\">0.46</gr_nbVal>");
    oOut.write("<gr_nbVal species=\"Amabilis_Fir\">0.18</gr_nbVal>");
    oOut.write("<gr_nbVal species=\"Subalpine_Fir\">0.28</gr_nbVal>");
    oOut.write("</gr_nciBeta>");
    oOut.write("<gr_nciWestern_HemlockNeighborLambda>");
    oOut.write("<gr_nlVal species=\"Western_Hemlock\">0.66</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Western_Redcedar\">0.37</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Amabilis_Fir\">0.40</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Subalpine_Fir\">0.43</gr_nlVal>");
    oOut.write("</gr_nciWestern_HemlockNeighborLambda>");
    oOut.write("<gr_nciWestern_RedcedarNeighborLambda>");
    oOut.write("<gr_nlVal species=\"Western_Hemlock\">0.44</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Western_Redcedar\">0.49</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Amabilis_Fir\">4.58E-4</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Subalpine_Fir\">5.58E-4</gr_nlVal>");
    oOut.write("</gr_nciWestern_RedcedarNeighborLambda>");
    oOut.write("<gr_nciAmabilis_FirNeighborLambda>");
    oOut.write("<gr_nlVal species=\"Western_Hemlock\">0.41</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Western_Redcedar\">6.21E-4</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Amabilis_Fir\">0.21</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Subalpine_Fir\">0.25</gr_nlVal>");
    oOut.write("</gr_nciAmabilis_FirNeighborLambda>");
    oOut.write("<gr_nciSubalpine_FirNeighborLambda>");
    oOut.write("<gr_nlVal species=\"Western_Hemlock\">0.41</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Western_Redcedar\">6.21E-4</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Amabilis_Fir\">0.21</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Subalpine_Fir\">0.18</gr_nlVal>");
    oOut.write("</gr_nciSubalpine_FirNeighborLambda>");
    oOut.write("<gr_nciHybrid_SpruceNeighborLambda>");
    oOut.write("<gr_nlVal species=\"Western_Hemlock\">0.41</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Western_Redcedar\">6.21E-4</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Amabilis_Fir\">0.27</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Subalpine_Fir\">0.22</gr_nlVal>");
    oOut.write("</gr_nciHybrid_SpruceNeighborLambda>");
    oOut.write("<gr_nciLodgepole_PineNeighborLambda>");
    oOut.write("<gr_nlVal species=\"Western_Hemlock\">0.48</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Western_Redcedar\">6.21</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Amabilis_Fir\">0.52</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Subalpine_Fir\">0.18</gr_nlVal>");
    oOut.write("</gr_nciLodgepole_PineNeighborLambda>");
    oOut.write("<gr_nciTrembling_AspenNeighborLambda>");
    oOut.write("<gr_nlVal species=\"Western_Hemlock\">0.38</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Western_Redcedar\">0.61</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Amabilis_Fir\">0.38</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Subalpine_Fir\">0.78</gr_nlVal>");
    oOut.write("</gr_nciTrembling_AspenNeighborLambda>");
    oOut.write("<gr_nciBlack_CottonwoodNeighborLambda>");
    oOut.write("<gr_nlVal species=\"Western_Hemlock\">0.33</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Western_Redcedar\">0.66</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Amabilis_Fir\">0.31</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Subalpine_Fir\">0.87</gr_nlVal>");
    oOut.write("</gr_nciBlack_CottonwoodNeighborLambda>");
    oOut.write("<gr_nciPaper_BirchNeighborLambda>");
    oOut.write("<gr_nlVal species=\"Western_Hemlock\">0.74</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Western_Redcedar\">0.19</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Amabilis_Fir\">0.14</gr_nlVal>");
    oOut.write("<gr_nlVal species=\"Subalpine_Fir\">0.88</gr_nlVal>");
    oOut.write("</gr_nciPaper_BirchNeighborLambda>");
    
    oOut.write("<gr_juvNCIMaxCrowdingRadius>");
    oOut.write("<gr_jnmcrVal species=\"Western_Redcedar\">16</gr_jnmcrVal>");
    oOut.write("<gr_jnmcrVal species=\"Trembling_Aspen\">17</gr_jnmcrVal>");
    oOut.write("</gr_juvNCIMaxCrowdingRadius>");
    oOut.write("<gr_juvNCIAlpha>");
    oOut.write("<gr_jnaVal species=\"Western_Redcedar\">2.34</gr_jnaVal>");
    oOut.write("<gr_jnaVal species=\"Trembling_Aspen\">2.35</gr_jnaVal>");
    oOut.write("</gr_juvNCIAlpha>");
    oOut.write("<gr_juvNCIBeta>");
    oOut.write("<gr_jnbVal species=\"Western_Redcedar\">0.42</gr_jnbVal>");
    oOut.write("<gr_jnbVal species=\"Trembling_Aspen\">0.57</gr_jnbVal>");
    oOut.write("</gr_juvNCIBeta>");
    oOut.write("<gr_juvNCIMaxPotentialGrowth>");
    oOut.write("<gr_jnmpgVal species=\"Western_Redcedar\">1.1</gr_jnmpgVal>");
    oOut.write("<gr_jnmpgVal species=\"Trembling_Aspen\">1.2</gr_jnmpgVal>");
    oOut.write("</gr_juvNCIMaxPotentialGrowth>");
    oOut.write("<gr_juvNCISizeEffectA>");
    oOut.write("<gr_jnseaVal species=\"Western_Redcedar\">0.03</gr_jnseaVal>");
    oOut.write("<gr_jnseaVal species=\"Trembling_Aspen\">0.085</gr_jnseaVal>");
    oOut.write("</gr_juvNCISizeEffectA>");
    oOut.write("<gr_juvNCISizeEffectB>");
    oOut.write("<gr_jnsebVal species=\"Western_Redcedar\">0.05</gr_jnsebVal>");
    oOut.write("<gr_jnsebVal species=\"Trembling_Aspen\">0.0236</gr_jnsebVal>");
    oOut.write("</gr_juvNCISizeEffectB>");
    oOut.write("<gr_juvNCICrowdingSlope>");
    oOut.write("<gr_jncslVal species=\"Western_Redcedar\">10.698</gr_jncslVal>");
    oOut.write("<gr_jncslVal species=\"Trembling_Aspen\">10.457</gr_jncslVal>");
    oOut.write("</gr_juvNCICrowdingSlope>");
    oOut.write("<gr_juvNCICrowdingSteepness>");
    oOut.write("<gr_jncstVal species=\"Western_Redcedar\">-10.00163</gr_jncstVal>");
    oOut.write("<gr_jncstVal species=\"Trembling_Aspen\">-10.00126</gr_jncstVal>");
    oOut.write("</gr_juvNCICrowdingSteepness>");
    oOut.write("<gr_juvNCIWestern_RedcedarNeighborLambda>");
    oOut.write("<gr_jnlVal species=\"Western_Redcedar\">0.1</gr_jnlVal>");
    oOut.write("<gr_jnlVal species=\"Trembling_Aspen\">0.2</gr_jnlVal>");
    oOut.write("</gr_juvNCIWestern_RedcedarNeighborLambda>");
    oOut.write("<gr_juvNCITrembling_AspenNeighborLambda>");
    oOut.write("<gr_jnlVal species=\"Western_Redcedar\">0.3</gr_jnlVal>");
    oOut.write("<gr_jnlVal species=\"Trembling_Aspen\">0.4</gr_jnlVal>");
    oOut.write("</gr_juvNCITrembling_AspenNeighborLambda>");
    
    
    oOut.write("<gr_juvNCIWestern_HemlockNeighborLambda>");
    oOut.write("<gr_jnlVal species=\"Western_Redcedar\">0.11</gr_jnlVal>");
    oOut.write("<gr_jnlVal species=\"Trembling_Aspen\">0.12</gr_jnlVal>");
    oOut.write("</gr_juvNCIWestern_HemlockNeighborLambda>");
    oOut.write("<gr_juvNCIWestern_RedcedarNeighborLambda>");
    oOut.write("<gr_jnlVal species=\"Western_Redcedar\">0.13</gr_jnlVal>");
    oOut.write("<gr_jnlVal species=\"Trembling_Aspen\">0.14</gr_jnlVal>");
    oOut.write("</gr_juvNCIWestern_RedcedarNeighborLambda>");
    oOut.write("<gr_juvNCIAmabilis_FirNeighborLambda>");
    oOut.write("<gr_jnlVal species=\"Western_Redcedar\">0.15</gr_jnlVal>");
    oOut.write("<gr_jnlVal species=\"Trembling_Aspen\">0.16</gr_jnlVal>");
    oOut.write("</gr_juvNCIAmabilis_FirNeighborLambda>");
    oOut.write("<gr_juvNCISubalpine_FirNeighborLambda>");
    oOut.write("<gr_jnlVal species=\"Western_Redcedar\">0.17</gr_jnlVal>");
    oOut.write("<gr_jnlVal species=\"Trembling_Aspen\">0.18</gr_jnlVal>");
    oOut.write("</gr_juvNCISubalpine_FirNeighborLambda>");
    oOut.write("<gr_juvNCIHybrid_SpruceNeighborLambda>");
    oOut.write("<gr_jnlVal species=\"Western_Redcedar\">0.19</gr_jnlVal>");
    oOut.write("<gr_jnlVal species=\"Trembling_Aspen\">0.2</gr_jnlVal>");
    oOut.write("</gr_juvNCIHybrid_SpruceNeighborLambda>");
    oOut.write("<gr_juvNCILodgepole_PineNeighborLambda>");
    oOut.write("<gr_jnlVal species=\"Western_Redcedar\">0.21</gr_jnlVal>");
    oOut.write("<gr_jnlVal species=\"Trembling_Aspen\">0.22</gr_jnlVal>");
    oOut.write("</gr_juvNCILodgepole_PineNeighborLambda>");
    oOut.write("<gr_juvNCITrembling_AspenNeighborLambda>");
    oOut.write("<gr_jnlVal species=\"Western_Redcedar\">0.23</gr_jnlVal>");
    oOut.write("<gr_jnlVal species=\"Trembling_Aspen\">0.24</gr_jnlVal>");
    oOut.write("</gr_juvNCITrembling_AspenNeighborLambda>");
    oOut.write("<gr_juvNCIBlack_CottonwoodNeighborLambda>");
    oOut.write("<gr_jnlVal species=\"Western_Redcedar\">0.25</gr_jnlVal>");
    oOut.write("<gr_jnlVal species=\"Trembling_Aspen\">0.26</gr_jnlVal>");
    oOut.write("</gr_juvNCIBlack_CottonwoodNeighborLambda>");
    oOut.write("<gr_juvNCIPaper_BirchNeighborLambda>");
    oOut.write("<gr_jnlVal species=\"Western_Redcedar\">0.27</gr_jnlVal>");
    oOut.write("<gr_jnlVal species=\"Trembling_Aspen\">0.28</gr_jnlVal>");
    oOut.write("</gr_juvNCIPaper_BirchNeighborLambda>");
    
    oOut.write("<gr_juvNCIMinNeighborDiam10>");
    oOut.write("<gr_jnmndVal species=\"Western_Redcedar\">5.3</gr_jnmndVal>");
    oOut.write("<gr_jnmndVal species=\"Trembling_Aspen\">5.5</gr_jnmndVal>");
    oOut.write("</gr_juvNCIMinNeighborDiam10>");
    oOut.write("<gr_juvNCIDiam10Divisor>50</gr_juvNCIDiam10Divisor>");
    oOut.write("<gr_juvNCIIncludeSnagsInNCI>0</gr_juvNCIIncludeSnagsInNCI>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }
}
